;################################################################################
;#										#
;# libgps - NMEA read library							#
;# copyright (c) 2008 Joerg Wolfram (joerg@jcwolfram.de)			#
;#										#
;# This library is free software; you can redistribute it and/or		#
;# modify it under the terms of the GNU Lesser General Public			#
;# License as published by the Free Software Foundation; either			#
;# version 2 of the License, or (at your option) any later version.		#
;#										#
;# This library is distributed in the hope that it will be useful,		#
;# but WITHOUT ANY WARRANTY; without even the implied warranty of		#
;# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the GNU		#
;# Lesser General Public License for more details.				#
;#										#
;# You should have received a copy of the GNU Lesser General Public		#
;# License along with this library; if not, write to the			#
;# Free Software Foundation, Inc., 59 Temple Place - Suite 330,			#
;# Boston, MA 02111-1307, USA.							#
;#										#
;################################################################################

;------------------------------------------------------------------------------
; DEC->BIN
; X points to string (long -> to zero byte before)++
; Y points to buffer (HI->LO) Hpos,Lpos
;------------------------------------------------------------------------------
libgps_decbin:	push	tempreg1
		push	tempreg2
		push	tempreg3
		push	r12
		push	r13
		push	r14
		push	r15
		mov	r12,const_0			;clear result
		mov	r13,const_0			;clear
		mov	r14,const_0			;clear result
		mov	r15,const_0			;clear result
		ldi	ZL,LOW(libgps_decf*2)		;pointer to table
		ldi	ZH,HIGH(libgps_decf*2)		

libgps_decbin1:	ldi	tempreg2,8			;number of digits
libgps_decbin2:	ld	tempreg1,X+	
		cpi	tempreg1,'.'			;ignore decimal point
		breq	libgps_decbin2
		cpi	tempreg1,0x31
		brcs	libgps_decbin4			;nothing to do
		subi	tempreg1,0x30			
libgps_decbin3:	lpm	tempreg3,Z+
		add	r12,tempreg3
		lpm	tempreg3,Z+
		adc	r13,tempreg3
		lpm	tempreg3,Z+
		adc	r14,tempreg3
		adc	r15,const_0
		sbiw	ZL,3
		dec	tempreg1
		brne	libgps_decbin3

libgps_decbin4:	adiw	ZL,3				;next multiplier
		dec	tempreg2
		brne	libgps_decbin2
		st	Y+,r15				;store result
		st	Y+,r14
		st	Y+,r13
		st	Y+,r12
libgps_decbin9:	pop	r15				;restore registers
		pop	r14
		pop	r13
		pop	r12
		pop	tempreg3
		pop	tempreg2
		pop	tempreg1
		ret			


;------------------------------------------------------------------------------
; BIN->DEC
; X points to string start
; Y points to buffer (HI->LO)
;------------------------------------------------------------------------------
libgps_bindec:	push	tempreg1
		push	tempreg2
		push	tempreg3
		push	r12
		push	r13
		push	r14
		push	r15
		ld	r12,Y+				;get bin value
		ldd	r13,Y+1		
		ldd	r14,Y+2
		ldd	r15,Y+3				;clear result
		ldi	ZL,LOW(libgps_decf*2)		;pointer to table
		ldi	ZH,HIGH(libgps_decf*2)		

		ldi	tempreg2,8			;number of digits
libgps_bindec1:	ldi	tempreg1,0x30			;0
libgps_bindec2:	lpm	tempreg3,Z+
		sub	r12,tempreg3
		lpm	tempreg3,Z+
		sbc	r13,tempreg3
		lpm	tempreg3,Z+
		sbc	r14,tempreg3
		sbc	r15,const_0
		brcs	libgps_bindec3
		sbiw	ZL,3
		inc	tempreg1
		rjmp	libgps_bindec2

libgps_bindec3:	sbiw	ZL,3
		lpm	tempreg3,Z+
		add	r12,tempreg3
		lpm	tempreg3,Z+
		adc	r13,tempreg3
		lpm	tempreg3,Z+
		adc	r14,tempreg3
		adc	r15,const_0

		cpi	tempreg2,5
		brne	libgps_bindec4
		ldi	tempreg3,'.'			;sep
		st	X+,tempreg3
libgps_bindec4:	st	X+,tempreg1
		dec	tempreg2
		brne	libgps_bindec1					
		rjmp	libgps_decbin9			;end


;------------------------------------------------------------------------------
; factors
;------------------------------------------------------------------------------
libgps_decf:	.db	0x80,0x96,0x98,0x40,0x42,0x0f	;1e7/1e6
		.db	0xa0,0x86,0x01,0x10,0x27,0x00	;1e5/1e4
		.db	0xe8,0x03,0x00,0x64,0x00,0x00	;1e3/1e2
		.db	0x0a,0x00,0x00,0x01,0x00,0x00	;1e1/1e0 							



