;################################################################################
;#										#
;# libmio - multi i/o for ATMega644 version 0.50m				#
;# normal video out								#
;# copyright (c) 2007-2008 Joerg Wolfram (joerg@jcwolfram.de)			#
;#										#
;# This library is free software; you can redistribute it and/or		#
;# modify it under the terms of the GNU Lesser General Public			#
;# License as published by the Free Software Foundation; either			#
;# version 2 of the License, or (at your option) any later version.		#
;#										#
;# This library is distributed in the hope that it will be useful,		#
;# but WITHOUT ANY WARRANTY; without even the implied warranty of		#
;# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the GNU		#
;# Lesser General Public License for more details.				#
;#										#
;# You should have received a copy of the GNU Lesser General Public		#
;# License along with this library; if not, write to the			#
;# Free Software Foundation, Inc., 59 Temple Place - Suite 330,			#
;# Boston, MA 02111-1307, USA.							#
;#										#
;################################################################################
;------------------------------------------------------------------------------
; dual mode (BAS/LCD) out
;------------------------------------------------------------------------------
libmio_vout:	ldi	YL,LOW(libmio_vram)	;2 base for LCD
		ldi	YH,HIGH(libmio_vram)	;2 base for LCD
		lds	XL,libmio_lcdpage	;2 page number
		ldi	XH,128			;1 bytes per page
		mul	XH,XL			;2
		add	YL,r0			;1 add page offset
		adc	YH,r1			;1
		lds	XL,libmio_lcdrow	;2
		ldi	ZH,0x80			;1 factor for row 0
		sbrc	XL,2			;1 row bit 2
		swap	ZH			;1 shift factor 4 bits right
		sbrc	XL,1			;1 row bit 1
		lsr	ZH			;1 shift factor 1 bit right
		sbrc	XL,1			;1 row bit 1
		lsr	ZH			;1 shift factor 1 bit right		
		sbrc	XL,0			;1 row bit 0
		lsr	ZH			;1 shift factor 1 bit right		
		lsr	XL			;1 /8
		lsr	XL			;1
		lsr	XL			;1
		andi	XL,0x07			;1 mask bits
		add	YL,XL			;1 add row offset
		adc	YH,const_0		;1
	
		mov	XL,vline_h
		cpi	XL,1
		brcc	libmio_vout_1
		mov	XL,vline_l
		cpi	XL,192
		brcs	libmio_vout_2
libmio_vout_1:	rjmp	libmio_vout_10		
			
libmio_vout_2:	lds	XL,libmio_rampos_l
		lds	XH,libmio_rampos_h
				
		ld	tempreg2,X+		;1 get pixline
		in	tempreg1,vport		;1 get outmask
		ldi	tempreg3,16		;1 8 bytes to do
		mov	tempreg7,const_0	;1 clear lcdbyte1
		mov	tempreg8,const_0	;1 clear lcdbyte2	
		
libmio_vout_3:	nop
		bst	tempreg2,0		;1 get px 7
		bld	tempreg1,vidp		;1 copy to outmask				
		out	vport,tempreg1		;1 output pix
	
		ld	ZL,Y			;2 get LCD byte 1
		adiw	YL,8			;2 set pointer to other page				
		bst	tempreg2,1		;1 get px 6
		bld	tempreg1,vidp		;1 copy to outmask				
		out	vport,tempreg1		;1 output pix	

		mul	ZL,ZH			;2 get shifted bit to R0.7
		nop				;1 filling
		nop				;1 filling
		bst	tempreg2,2		;1 get px 5
		bld	tempreg1,vidp		;1 copy to outmask				
		out	vport,tempreg1		;1 output pix	

		lsl	r0			;1 shift out bit
		nop				;1 filling
		nop				;1 filling
		nop				;1 filling		
		bst	tempreg2,3		;1 get px 4
		bld	tempreg1,vidp		;1 copy to outmask				
		out	vport,tempreg1		;1 output pix	

		sbrs	tempreg3,0		;1 skip if right side
		ror	tempreg7		;1 shift bit to lcdbyte 1
		nop				;1 filling
		nop				;1 filling						
		bst	tempreg2,4		;1 get px 3
		bld	tempreg1,vidp		;1 copy to outmask				
		out	vport,tempreg1		;1 output pix	

		sbrc	tempreg3,0		;1 skip if left side
		ror	tempreg8		;1 shift bit to lcdbyte 2
		nop				;1 filling
		nop				;1 filling						
		bst	tempreg2,5		;1 get px 2
		bld	tempreg1,vidp		;1 copy to outmask				
		out	vport,tempreg1		;1 output pix	

		nop				;1 filling
		nop				;1 filling
		nop				;1 filling
		nop				;1 filling		
		bst	tempreg2,6		;1 get px 1
		bld	tempreg1,vidp		;1 copy to outmask				
		out	vport,tempreg1		;1 output pix	

		nop				;1 filling
		nop				;1 filling		
		bst	tempreg2,7		;1 get px 0
		ld	tempreg2,X+		;2 get next pitempreg2ine				
		bld	tempreg1,vidp		;1 copy to outmask				
		out	vport,tempreg1		;1 output pix			
		
		dec	tempreg3		;1
		brne	libmio_vout_3		;2
						
		sbiw	XL,1			;2 correct pointer
		clt				;1 set to zero
		bld	tempreg1,vidp		;1 output pix
		out	vport,tempreg1		;1
		lds	YL,libmio_cline
		inc	YL
		cpi	YL,3
		brcs	libmio_vout_4
		clr	YL			;1 set cline to 0
		sts	libmio_rampos_l,XL	;2 store video position
		sts	libmio_rampos_h,XH			
libmio_vout_4:	sts	libmio_cline,YL		
		rjmp	libmio_vout_20
		
libmio_vout_10:	ldi	tempreg3,16		;1 8 bytes to do
		mov	tempreg7,const_0	;1 clear lcdbyte1
		mov	tempreg8,const_0	;1 clear lcdbyte2		
libmio_vout_11:	ld	ZL,Y			;2 get LCD byte 1
		adiw	YL,8			;2 set pointer to other page			
		mul	ZL,ZH			;2 get shifted bit to R0.7		
		lsl	r0			;1 shift out bit		
		sbrs	tempreg3,0		;1 skip if right side
		ror	tempreg7		;1 shift bit to lcdbyte 1
		sbrc	tempreg3,0		;1 skip if left side
		ror	tempreg8		;1 shift bit to lcdbyte 2		
		dec	tempreg3		;1
		brne	libmio_vout_11		;1
		
libmio_vout_20:	lds	XL,libmio_lcdrow	;2 get row
		cpi	XL,64			;1 check if <64	
		brcs	libmio_vout_23		;1 no
		rjmp	libmio_vout_40		;1 no
libmio_vout_23:	sbi	lcdctrl,lcd_cs1		;2 chip select 1 active
		cbi	lcdctrl,lcd_cs2		;2 chip select 2 inactive
		sbi	lcdctrl,lcd_di		;2 data mode

		sbrs	tempreg7,0		
		cbi	lcdp_0,lcdbit0		;copy bits 0+1
		sbrs	tempreg7,1		
		cbi	lcdp_1,lcdbit1	
		sbrc	tempreg7,0
		sbi	lcdp_0,lcdbit0
		sbrc	tempreg7,1
		sbi	lcdp_1,lcdbit1
		inc	XL			;1 row+1
		sts	libmio_lcdrow,XL	;2 store
		mov	XL,tempreg7
		andi	XL,0xfc			;clear bits 0+1
		out	lcddata,XL		;1 Data byte for controller1
		
		ldi	YL,0x00			;1 keycode=0
		sbis	K1port,key_ent		;1 skip if Enter is not pressed
		ori	YL,0x20			;1 key code for ENTER
		sbis	K1port,key_esc		;1 skip if Esc is not pressed
		ori	YL,0x10			;1 key code for ESC			
		sbis	K2port,key_up		;1 skip if UP is not pressed
		ori	YL,0x04			;1 key code for UP
		sbis	K2port,key_down		;1 skip if DOWN is not pressed
		ori	YL,0x08			;1 key code for DOWN		
		sbis	K2port,key_left		;1 skip if LEFT is not pressed
		ori	YL,0x01			;1 key code for LEFT
		sbis	K2port,key_right	;1 skip if RIGHT is not pressed
		ori	YL,0x02			;1 key code for RIGHT
		sts	libmio_keybits,YL	;2 store bitvector

		sbi	lcdctrl,lcd_e		;2 raise clock
		
		ldi	YL,0x00			;1 keycode=0
		sbis	K1port,key_ent		;1 skip if Enter is not pressed
		ldi	YL,0xea			;1 key code for ENTER
		sbis	K1port,key_esc		;1 skip if Esc is not pressed
		ldi	YL,0xed			;1 key code for ESC			
		sbis	k2port,key_up		;1 skip if UP is not pressed
		ldi	YL,0xe4			;1 key code for UP
		sbis	k2port,key_down		;1 skip if DOWN is not pressed
		ldi	YL,0xe5			;1 key code for DOWN		
		sbis	k2port,key_left		;1 skip if LEFT is not pressed
		ldi	YL,0xe2			;1 key code for LEFT
		sbis	k2port,key_right	;1 skip if RIGHT is not pressed
		ldi	YL,0xe3			;1 key code for RIGHT		
		
		sts	libmio_keycode,YL	;2 store keycode
		cpi	YL,0x00			;1 is zero
		breq	libmio_vout_21		;1/2 yes
		sts	libmio_lastkey,YL	;2 store code for last key
libmio_vout_21:		
				
		cbi	lcdctrl,lcd_e		;2 lower clock	
		nop				;1 adr/data hold
		nop				;1
		nop				;1
		nop				;1
		nop				;1
		nop				;1
		nop				;1		
		cbi	lcdctrl,lcd_cs1		;2 chip select 1 inactive
		sbi	lcdctrl,lcd_cs2		;2 chip select 2 active

		sbrs	tempreg8,0
		cbi	lcdp_0,lcdbit0		;copy bits 0+1
		sbrs	tempreg8,1
		cbi	lcdp_1,lcdbit1	
		sbrc	tempreg8,0
		sbi	lcdp_0,lcdbit0
		sbrc	tempreg8,1
		sbi	lcdp_1,lcdbit1
		mov	XL,tempreg8
		andi	XL,0xfc			;clear bits 0+1
				
		out	lcddata,XL		;1 Data byte for controller2
		sbi	lcdctrl,lcd_e		;2 raise clock
		
		ldi	YL,0x00			;1 keystat=0
		sbis	K1port,key_ent		;1 skip if Enter is not pressed
		ori	YL,0x04			;1 left ctrl
		sbis	K1port,key_esc		;1 skip if Esc is not pressed
		ori	YL,0x01			;1 left shift			
		sbis	k2port,key_up		;1 skip if UP is not pressed
		ori	YL,0x02			;1 right shift
		sbis	k2port,key_down		;1 skip if DOWN is not pressed
		ori	YL,0x08			;1 right ctrl		
		sts	libmio_kbdstate,YL	;2
		
		cbi	lcdctrl,lcd_e		;2 lower clock			
		rjmp	libmio_vout_e		;2 end
				
		
libmio_vout_40:	brne	libmio_vout_50		;not 64
		sbi	lcdctrl,lcd_cs1		;2 chip select 1 active
		sbi	lcdctrl,lcd_cs2		;2 chip select 2 active		
		cbi	lcdctrl,lcd_di		;2 instruction mode
		ldi	XH,0x40			;1 Y address=0
		cbi	lcdp_0,lcdbit0		;copy bits 0+1		
		cbi	lcdp_1,lcdbit1	
		out	lcddata,XH		;1 set out
		inc	XL			;1 row+1
		sts	libmio_lcdrow,XL	;2 store
		
		ldi	YL,0x00			;1 keycode=0
		sbis	K1port,key_ent		;1 skip if Enter is not pressed
		ldi	YL,0xea			;1 key code for ENTER
		sbis	K1port,key_esc		;1 skip if Esc is not pressed
		ldi	YL,0xed			;1 key code for ESC			
		sbis	k2port,key_up		;1 skip if UP is not pressed
		ldi	YL,0xe4			;1 key code for UP
		sbis	k2port,key_down		;1 skip if DOWN is not pressed
		ldi	YL,0xe5			;1 key code for DOWN		
		sbis	k2port,key_left		;1 skip if LEFT is not pressed
		ldi	YL,0xe2			;1 key code for LEFT
		sbis	k2port,key_right	;1 skip if RIGHT is not pressed
		ldi	YL,0xe3			;1 key code for RIGHT
		
		
		sts	libmio_keycode,YL	;2 store keycode
		cpi	YL,0x00			;1 is zero
		breq	libmio_vout_41		;1/2 yes
		sts	libmio_lastkey,YL	;2 store code for last key
libmio_vout_41:		
				
		sbi	lcdctrl,lcd_e		;2 raise clock				

		ldi	YL,0x00			;1 keystat=0
		sbis	K1port,key_ent		;1 skip if Enter is not pressed
		ori	YL,0x04			;1 left ctrl
		sbis	K1port,key_esc		;1 skip if Esc is not pressed
		ori	YL,0x01			;1 left shift			
		sbis	k2port,key_up		;1 skip if UP is not pressed
		ori	YL,0x02			;1 right shift
		sbis	k2port,key_down		;1 skip if DOWN is not pressed
		ori	YL,0x08			;1 right ctrl		
		sts	libmio_kbdstate,YL	;2
		
		cbi	lcdctrl,lcd_e		;2 lower clock			
		rjmp	libmio_vout_e		;2 end	
		
libmio_vout_50:	sbi	lcdctrl,lcd_cs1		;2 chip select 1 active
		sbi	lcdctrl,lcd_cs2		;2 chip select 2 active		
		cbi	lcdctrl,lcd_di		;2 instruction mode		
		lds	XH,libmio_lcdpage	;2 get page
		andi	XH,0x07			;1
		ori	XH,0xb8			;1
		sbrs	XH,0		
		cbi	lcdp_0,lcdbit0		;copy bits 0+1
		sbrs	XH,1		
		cbi	lcdp_1,lcdbit1	
		sbrc	XH,0
		sbi	lcdp_0,lcdbit0
		sbrc	XH,1
		sbi	lcdp_1,lcdbit1
		mov	XL,XH
		andi	XL,0xfc			;clear bits 0+1
		out	lcddata,XL		;1
		inc	XH			;1 page +1
		andi	XH,0x07			;1 mask bits
		sts	libmio_lcdpage,XH	;2 store back page
		sts	libmio_lcdrow,const_0	;2 set address to zero
		
		ldi	YL,0x00			;1 keycode=0
		sbis	K1port,key_ent		;1 skip if Enter is not pressed
		ldi	YL,0xea			;1 key code for ENTER
		sbis	K1port,key_esc		;1 skip if Esc is not pressed
		ldi	YL,0xed			;1 key code for ESC			
		sbis	K2port,key_up		;1 skip if UP is not pressed
		ldi	YL,0xe4			;1 key code for UP
		sbis	K2port,key_down		;1 skip if DOWN is not pressed
		ldi	YL,0xe5			;1 key code for DOWN		
		sbis	K2port,key_left		;1 skip if LEFT is not pressed
		ldi	YL,0xe2			;1 key code for LEFT
		sbis	K2port,key_right	;1 skip if RIGHT is not pressed
		ldi	YL,0xe3			;1 key code for RIGHT
		
		
		sts	libmio_keycode,YL	;2 store keycode
		cpi	YL,0x00			;1 is zero
		breq	libmio_vout_51		;1/2 yes
		sts	libmio_lastkey,YL	;2 store code for last key
libmio_vout_51:		
				
		sbi	lcdctrl,lcd_e		;2 raise clock				

		ldi	YL,0x00			;1 keystat=0
		sbis	K1port,key_ent		;1 skip if Enter is not pressed
		ori	YL,0x04			;1 left ctrl
		sbis	K1port,key_esc		;1 skip if Esc is not pressed
		ori	YL,0x01			;1 left shift			
		sbis	k2port,key_up		;1 skip if UP is not pressed
		ori	YL,0x02			;1 right shift
		sbis	k2port,key_down		;1 skip if DOWN is not pressed
		ori	YL,0x08			;1 right ctrl		
		sts	libmio_kbdstate,YL	;2
		
		cbi	lcdctrl,lcd_e		;2 lower clock			
		
libmio_vout_e:	ret				;end of vout routine -> VS/Frame handling				

		

				
		
																			