;################################################################################
;#										#
;# libmio - multi i/o for ATMega644 version 0.40				#
;# main library									#
;# copyright (c) 2005-2008 Joerg Wolfram (joerg@jcwolfram.de)			#
;#										#
;# This library is free software; you can redistribute it and/or		#
;# modify it under the terms of the GNU Lesser General Public			#
;# License as published by the Free Software Foundation; either			#
;# version 2 of the License, or (at your option) any later version.		#
;#										#
;# This library is distributed in the hope that it will be useful,		#
;# but WITHOUT ANY WARRANTY; without even the implied warranty of		#
;# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the GNU		#
;# Lesser General Public License for more details.				#
;#										#
;# You should have received a copy of the GNU Lesser General Public		#
;# License along with this library; if not, write to the			#
;# Free Software Foundation, Inc., 59 Temple Place - Suite 330,			#
;# Boston, MA 02111-1307, USA.							#
;#										#
;################################################################################

;-------------------------------------------------------------------------------		
;video-interrupt (no registers destroyed, stack=13)
;save registers
;-------------------------------------------------------------------------------		
libmio_isr:	push	tempreg1		;2
		push	tempreg2		;2
		push	tempreg3		;2
		push	tempreg7
		push	tempreg8
		push	ZH			;2 save Z-register
		push	ZL			;2 save Z-register
		push	YH			;2 save Y-register
		push	YL			;2 save Y-register
		push	XH			;2 save Y-register
		push	XL			;2 save Y-register
		push	r0			;2 save r0
		push	r1			;2 save r1		
		in	ZH,SREG			;1 save status
		push	ZH			;2
	
		sbis	TIFR0,1			;1
		rjmp	libmio_isr_1
		sbi	TIFR0,1			;1 reset flag
		lds	XL,libmio_tstat		;2 timer status
		ori	XL,0x80
		sts	libmio_tstat,XL
libmio_isr_1:			
;-------------------------------------------------------------------------------
; set pointer for code saving
;-------------------------------------------------------------------------------
		add	vline_l,const_1
		adc	vline_h,const_0

;-------------------------------------------------------------------------------
;sync to counter
;-------------------------------------------------------------------------------		
libmio_iss:	lds	XL,TCNT1L		;1 get timer
		cpi	XL,libmio_hstart+60 	;1
		brcs	libmio_iss		;1 loop		
libmio_iss_01:	cpi	XL,libmio_hstart+61	
		brcs	libmio_iss_02
libmio_iss_02:	cpi	XL,libmio_hstart+62	
		brcs	libmio_iss_03
libmio_iss_03:	cpi	XL,libmio_hstart+63	
		brcs	libmio_iss_03a
libmio_iss_03a:	cpi	XL,libmio_hstart+64	
		brcs	libmio_iss_04
		
libmio_iss_04:	lds	XL,libmio_sysconf
		sbrs	XL,4
		call	libmio_vout		;normal??
		lds	XL,libmio_sysconf
		sbrc	XL,4
		call	libmio_uout		;rotated
		
		ldi	YL,LOW(libmio_ram)	;1 set ptr
		ldi	YH,HIGH(libmio_ram)	;1
				
;-------------------------------------------------------------------------------
; calculate sound (34c)
;-------------------------------------------------------------------------------
		ldd	tempreg2,Y+lmo_rand2	;2 noise signal
		ldd	XL,Y+lmo_wpos_l		;2 wave pointer low
		ldd	ZL,Y+lmo_wpos_h		;2 wave pointer high
		ldd	r0,Y+lmo_woff_l		;2 wave offset low
		ldd	r1,Y+lmo_woff_h		;2 wave offset high
		add	XL,r0			;1 calculate new pointer
		adc	ZL,r1			;1 
		std	Y+lmo_wpos_l,XL		;2 writeback new pointer
		std	Y+lmo_wpos_h,ZL		;2 
		ldd	XL,Y+lmo_note_val	;2 note val
		cpi	XL,0xff			;1
		brne	libmio_wa_01		;1/2
		mov	ZL,tempreg2	
libmio_wa_01:	ldd	tempreg2,Y+lmo_envv	;2 envelope value
		ldi	XH,0x00			;min/off		
		sbrs	ZL,7
		mov	XH,tempreg2		;level
		sts	OCR2A,XH		;1 set level
		
;-------------------------------------------------------------------------------
; pseudo-random generator (31C)
;-------------------------------------------------------------------------------
		ldd	ZL,Y+lmo_rand1		;2
		ldd	ZH,Y+lmo_rand2		;2
		ldd	tempreg1,Y+lmo_rand3	;2
		ldd	tempreg2,Y+lmo_rand4	;2
		clr	XL			;1
		sbrc	ZL,2			;1
		com	XL			;1
		sbrc	ZL,5			;1
		com	XL			;1
		sbrc	ZH,4			;1
		com	XL			;1
		sbrc	tempreg2,7		;1
		com	XL			;1
		rol	XL			;1
		rol	ZL			;1
		rol	ZH			;1
		rol	tempreg1		;1
		rol	tempreg2		;1
		std	Y+lmo_rand1,ZL		;2
		std	Y+lmo_rand2,ZH		;2
		std	Y+lmo_rand3,tempreg1	;2
		std	Y+lmo_rand4,tempreg2	;2		
				
;-------------------------------------------------------------------------------		
;check if line is image
;-------------------------------------------------------------------------------
		ldd	ZL,Y+lmo_sysconf	;2 configuration
		ldi	ZL,0xff
		sbrc	vline_h,0		;1 skip if less than 256
		rjmp	libmio_is_02		;2 no visible line
		ldi	ZH,192
libmio_ismode1:	cp	vline_l,ZH		;1 max lines
		brcc	libmio_is_02		;2 goto no image-line
		rjmp	libmio_is_15		;2 image line -> skip control		

;-------------------------------------------------------------------------------
;no image line
;-------------------------------------------------------------------------------			
libmio_is_02:	ldi	ZH,libmio_vstart_p	;1 vstart pal
		sbrs	ZL,0			;1 pal/ntsc bit
		ldi	ZH,libmio_vstart_n	;1 vstart ntsc
		cp	vline_l,ZH
		brne	libmio_is_09		;no vsync start
		ldi	XH,0x82			;invert CSYNC
		sts	TCCR1A,XH
		rjmp	libmio_is_15  


libmio_is_09:	ldi	ZH,libmio_vstop_p	;1 vstop pal
		sbrs	ZL,0			;1 pal/ntsc bit
		ldi	ZH,libmio_vstop_n	;1 vstop ntsc
		cp	vline_l,ZH
		brne	libmio_is_13		;no vsync stop
		ldi	XH,0xc2			;normal CSYNC
		sts	TCCR1A,XH
		rjmp	libmio_is_15  

libmio_is_13:	ldi	ZH,57			;1 maxlines pal
		sbrs	ZL,0			;1 pal/ntsc bit
		ldi	ZH,7			;1 maxlines ntsc
		cp	vline_l,ZH
		breq	libmio_is_14		;2 end of frame		
		rjmp	libmio_is_15  
		
libmio_is_14:	ldi	XL,LOW(libmio_vram)	;1 set vram start
		std	Y+lmo_rampos_l,XL	;2
		ldi	XL,HIGH(libmio_vram)	;1
		std	Y+lmo_rampos_h,XL	;2
		std	Y+lmo_cline,const_0	;2 store cline
		clr	vline_l			;line_l=0
		clr	vline_h			;line_h=0
;increment counters
		ldi	XL,5			;1 fps/10 for pal
		sbrs	ZL,0			;1 pal/ntsc bit
		inc	XL			;1 fps/10 for ntsc
		ldi	tempreg1,50		;1 fps for pal
		sbrs	ZL,0			;1 pal/ntsc bit
		ldi	tempreg1,60		;1 fps for ntsc
		ldd	XH,Y+lmo_time0		;2 timer
		inc	XH
		cp	XH,XL			;1 frames (5/6)
		brne	libmio_is_06a
		clr	XH
		ldd	ZL,Y+lmo_time_l
		ldd	ZH,Y+lmo_time_h
		adiw	ZL,1
		std	Y+lmo_time_l,ZL
		std	Y+lmo_time_h,ZH		
libmio_is_06a:	std	Y+lmo_time0,XH		
		ldd	ZL,Y+lmo_frames		;2 get frame counter
		inc	ZL			;1 +1
		cp	ZL,tempreg1		;1 frames (50/60)
		brne	libmio_is_07		;2 frame oveflow
		clr	ZL			;1 set frames=0
libmio_is_07:	std	Y+lmo_frames,ZL		;2 set frames=0

;-------------------------------------------------------------------------------
; calculate envelope position
;-------------------------------------------------------------------------------		
		ldd	XL,Y+lmo_env		;2 envelope pointer
		subi	XL,0x01			;1 +1
		brcc	libmio_is_11a		;end
		clr	XL
		clr	XH
libmio_is_11a:	std	Y+lmo_env,XL		;write back
		swap	XL
		std	Y+lmo_envv,XL		;write envelope value

;-------------------------------------------------------------------------------
; check sequencer
;-------------------------------------------------------------------------------		
libmio_is_seq:	lds	XL,libmio_seqspeed	;speed
		cpi	XL,0x00			;enabled?
		brne	libmio_is_seq1		;yes
libmio_is_seq0:	rjmp	libmio_is_seq7
libmio_is_seq1:	lds	XH,libmio_seqcount	;counter
		inc	XH
		sts	libmio_seqcount,XH
		cp	XH,XL
		brcs	libmio_is_seq0
		clr	XH
		sts	libmio_seqcount,XH
		lds	ZL,libmio_seql
		lds	ZH,libmio_seqh
		lds	YL,libmio_seqbl
		lds	YH,libmio_seqbh
		adiw	YL,1
		ld	XL,Z+
		sts	libmio_seql,ZL		;store back pointer
		sts	libmio_seqh,ZH
		cpi	XL,0xfe			;no tone
		breq	libmio_is_seq2
		rcall	libmio_note
libmio_is_seq2:	cp	ZH,YH			;compare hi
		brcs	libmio_is_seq7		;is less				
		cp	ZL,YL			;compare low
		brcs	libmio_is_seq7		;is less
		lds	XL,libmio_seqstat
		cpi	XL,0x01			;stop required
		brne	libmio_is_seq6		;no
		ldi	XL,0x02			;stop status
		sts	libmio_seqspeed,const_0	;stop sequencer
		sts	libmio_seqstat,XL	;write stop status
						
libmio_is_seq6:	lds	ZL,libmio_seqal		;get start pointer
		lds	ZH,libmio_seqah
		sts	libmio_seql,ZL		;store pointer
		sts	libmio_seqh,ZH
		
libmio_is_seq7:
;-------------------------------------------------------------------------------
; keyboard repeat
;-------------------------------------------------------------------------------		
libmio_kr_01:	lds	ZL,libmio_keycode	;this keycode
		lds	ZH,libmio_oldkey	;last keycode
		sts	libmio_oldkey,ZL	;store this as last
		cp	ZL,ZH			;change?
		brne	libmio_kr_02
		cpi	ZL,0			;no key is pressed
		breq	libmio_kr_02		;is zero
		lds	ZL,libmio_krc		;get repeat counter
		inc	ZL			;+1
		sts	libmio_krc,ZL		;write back
		cpi	ZL,32
		brne	libmio_kr_en
		sts	libmio_keycode,const_0
		ldi	ZL,26
		sts	libmio_krc,ZL
		rjmp	libmio_kr_en
libmio_kr_02:	sts	libmio_krc,const_0
libmio_kr_en:	

libmio_is_15:		

;libmio_tstat   0=stop  1=over	2=run  40=reset				
;timer routine
libmio_itim:	lds	XL,libmio_tstat		;2 get timer status
		sbrs	XL,7			;1 overflow?
		rjmp	libmio_itim_e		;no
		andi	XL,0x7f			;1 clear overflow bit
		sts	libmio_tstat,XL		;2 write timer status
		cpi	XL,0x40			;1 clear
		brne	libmio_itim_1		;1/2 no
		sts	libmio_tmsl,const_0	;2 clear counters
		sts	libmio_tmsh,const_0	;2
		sts	libmio_tsl,const_0	;2
		sts	libmio_tsh,const_0	;2
		sts	libmio_tpre,const_0	;2
		sts	libmio_tstat,const_0	;2 stop mode		
		rjmp	libmio_itim_e		;2 goto end
libmio_itim_1:	cpi	XL,0x02			;1 stop	
		brcs	libmio_itim_e		;1/2 do nothing if stop
		lds	XL,libmio_tpre		;2 get prescaler
		inc	XL			;1 +1
		sts	libmio_tpre,XL		;2 store prescaler
		cpi	XL,0x0a			;1 10?
		brcs	libmio_itim_e		;1/2 less		
		sts	libmio_tpre,const_0	;2 reset prescaler
		lds	XL,libmio_tmsl		;2 get low ms
		lds	XH,libmio_tmsh		;2 get high ms
		adiw	XL,1			;2 ms+1
		sts	libmio_tmsl,XL		;2
		sts	libmio_tmsh,XH		;2	
		cpi	XH,HIGH(1000)		;1
		brcs	libmio_itim_e		;1/2
		cpi	XL,low(1000)		;1
		brcs	libmio_itim_e		;1/2
		sts	libmio_tmsl,const_0	;2 clear msec counter
		sts	libmio_tmsh,const_0	;2	
		lds	XL,libmio_tsl		;2 get low ms
		lds	XH,libmio_tsh		;2 get high ms
		adiw	XL,1			;2 ms+1
		sts	libmio_tsl,XL		;2
		sts	libmio_tsh,XH		;2	
		cpi	XH,HIGH(32767)		;1
		brcs	libmio_itim_e		;1/2
		cpi	XL,low(32767)		;1		
		brcs	libmio_itim_e		;1/2
		sts	libmio_tstat,const_1	;2 overflow			
libmio_itim_e:					;end	

;-------------------------------------------------------------------------------
;key repeat and check for restart code
;-------------------------------------------------------------------------------		
libmio_is_20:	lds	ZL,libmio_keybits
		andi	ZL,0x3c
		cpi	ZL,0x3c
		brne	libmio_is_25
		jmp	0
		
;-------------------------------------------------------------------------------
;restore registers and exit
;-------------------------------------------------------------------------------				
libmio_is_25:	sbis	PINC,2			;skip if no screenshot
		call	mem_scrs		;do screenshot
		pop	ZH			;2 restore sreg
		out	SREG,ZH			;1 put back    
		pop	r1			;2 restore r1
		pop	r0			;2 restore r0
		pop	XL			;2 restore Y-register
		pop	XH			;2 restore Y-register
libmio_is_30:	pop	YL			;2 restore Y-register
		pop	YH			;2 restore Y-register
		pop	ZL			;2 restore Z-register
		pop	ZH			;2 restore Z-register
		pop	tempreg8
		pop	tempreg7
		pop	tempreg3		;
		pop	tempreg2		;2
		pop	tempreg1		;2 restore register
		reti				;4 thats all

;-------------------------------------------------------------------------------
;port konfiguration for port A ... D
;-------------------------------------------------------------------------------
libmio_init:	ldi	XL,0xfc			;port direction (lcddata)
		out	DDRA,XL
		ldi	XL,0x00
		out	PORTA,XL	

		ldi	XL,0xb0			;port direction (+SPI)
		out	DDRB,XL
		ldi	XL,0x0f
		out	PORTB,XL	


		ldi	XL,0xfb			;port direction (screenshot as input)
		out	DDRC,XL
		ldi	XL,0x14
		out	PORTC,XL	

		ldi	XL,0xf2			;set pin 1,4-7 as output
		out	DDRD,XL
		ldi	XL,0xfc
		out	PORTD,XL	
;-------------------------------------------------------------------------------
;timer configuration
;-------------------------------------------------------------------------------
		call	mem_getconf
		ldi	XH,0xc2			;FAST-PWM-Mode 14 (PWM channel B)
		sts	TCCR1A,XH
		ldi	XH,0x19			;FAST-PWM-Mode (no prescaler)
		sts	TCCR1B,XH
		lds	XL,libmio_sysconf
		ldi	ZH,HIGH(libmio_counter_p-1)
		sbrs	XL,0			;1 pal/ntsc bit
		ldi	ZH,HIGH(libmio_counter_n-1)
		sts	ICR1H,ZH
		ldi	ZH,LOW(libmio_counter_p-1)
		sbrs	XL,0			;1 pal/ntsc bit
		ldi	ZH,LOW(libmio_counter_n-1)
		sts	ICR1L,ZH

		sts	OCR1AH,const_0		;we use only 8 bits
		ldi	ZH,libmio_hslen		;1 hsync length pal
		sts	OCR1AL,ZH		

		ldi	ZH,HIGH(libmio_hstart)	;video start int
		sts	OCR1BH,ZH		
		ldi	ZH,LOW(libmio_hstart)	;1 hor position
		sts	OCR1BL,ZH		
				
		ldi	XL,0x04
		sts	TIMSK1,XL		;enable OC1B int

		ldi	XL,0x83			;timer 2 config
		sts	TCCR2A,XL
		ldi	XL,0x01			;timer 2 config
		sts	TCCR2B,XL
		ldi	XL,0x00			;zero signal
		sts	OCR2A,XL
		sts	ASSR,const_0

;init timer 0 for 10K
		ldi	XL,0x02
		out	TCCR0A,XL
		out	TCCR0B,XL
		ldi	XL,249
		out	OCR0A,XL

;-------------------------------------------------------------------------------
;uart configuration (async 9600Hz)
;-------------------------------------------------------------------------------
		ldi	XL,0x02			;set U2X
		sts	UCSR0A,XL
		ldi	XL,0x18			;
		sts	UCSR0B,XL
		ldi	XL,0x06			;
		sts	UCSR0C,XL
		ldi	XH,HIGH(259)
		ldi	XL,LOW(259)
		sts	UBRR0H,XH
		sts	UBRR0L,XL
		
;-------------------------------------------------------------------------------
;spi configuration (mode 3 master 5MHz/156kHz)
;-------------------------------------------------------------------------------
		ldi	XL,0x5c		;f/4
		lds	XH,libmio_sysconf
		sbrc	XH,1
		ldi	XL,0x5f		;f/128
		out	SPCR0,XL
		ldi	XL,0x00
		out	SPSR0,XL

;init LCD
		sbi	lcdctrl,lcd_cs1		;2 chip select 1 active
		sbi	lcdctrl,lcd_cs2		;2 chip select 2 active		
		cbi	lcdctrl,lcd_di		;2 instruction mode
		sbi	lcdp_0,lcdbit0
		sbi	lcdp_1,lcdbit1		
		ldi	XH,0x3c			;1 Display ON
		out	lcddata,XH		;1 set out
		ldi	XH,20
libmio_ilcd1:	dec	XH
		brne	libmio_ilcd1
		sbi	lcdctrl,lcd_e		;2 raise clock				
		ldi	XH,20
libmio_ilcd2:	dec	XH
		brne	libmio_ilcd2
		cbi	lcdctrl,lcd_e		;2 lower clock				
		ldi	XH,0
libmio_ilcd3:	rcall	libmio_return
		dec	XH
		brne	libmio_ilcd3
		ldi	XH,0xf8			;1 DIS address=0
		cbi	lcdp_0,lcdbit0
		cbi	lcdp_1,lcdbit1		
		out	lcddata,XH		;1 set out
		ldi	XH,20
libmio_ilcd4:	dec	XH
		brne	libmio_ilcd4
		sbi	lcdctrl,lcd_e		;2 raise clock							
		ldi	XH,20
libmio_ilcd5:	dec	XH
		brne	libmio_ilcd5
		cbi	lcdctrl,lcd_e		;2 lower clock				
		call	libmio_setconf		;configure i/o

;-------------------------------------------------------------------------------
;clear video-RAM and set pointers
;-------------------------------------------------------------------------------
		ldi	YL,LOW(libmio_ram)
		ldi	YH,HIGH(libmio_ram)
		ldi	XL,LOW(libmio_vram)	;2
		st	Y+,XL			;2 start vram
		ldi	XL,HIGH(libmio_vram)	;2
		st	Y+,XL			;2 start vram
		ldi	XL,0x1e
libmio_init_c:	st	Y+,const_0
	    	dec	XL
		brne	libmio_init_c	
		ldi	XL,0xaa	    
		sts	libmio_rand1,XL
		sts	libmio_rand3,XL
		sts	libmio_channel,const_0	;set output to screen
		sts	libmio_font,const_0	;standard font
		sts	libmio_env,const_0		;sound off
		sbi	lcdp_1,lcdbit1
						
libmio_vmode:	
libmio_vmodes:	
libmio_return:	ret    
		
;------------------------------------------------------------------------------
; set color to XL
;------------------------------------------------------------------------------
libmio_setcol:	andi	XL,1
		sts	libmio_color,XL
		ret		

;-------------------------------------------------------------------------------
; Bin16 -> ASCII (dez)
; XL,XH 	= number
; ctrl		= format
; 	Bit 0,1	 = format	00 = compact
;				01 = leading spaces
;				10 = leading zeroes 0...65536
;				11 = leading zeroes with +- sign  
;			
;	Bit 2,3	 = number of visible digits (2,3,4,5)
;	Bit 4,5	 = comma position (no  xxxx,x   xxx,xx  xx,xxx) 
;	Bit 6	1= big size 
;-------------------------------------------------------------------------------
.ifdef use_libmio_outd
libmio_outd:	push	tempreg1		;save registers
		push	tempreg2
		push	tempreg3
		push	XL
		push	XH
		ldi	tempreg1,0x20		;space
		mov	tempreg2,ctrl
		andi	tempreg2,0x03
		cpi	tempreg2,0x02		;leading zeroes without sign
		breq	libmio_outd_03
		cpi	tempreg2,0x03		;leading zeroes with +- sign
		brne	libmio_outd_01
		ldi	tempreg1,'+'		;set positive sign
libmio_outd_01:	sbrs	XH,7
		rjmp	libmio_outd_02
		com	XL			;X=-X
		com	XH			;
		add	XL,const_1		;
		adc	XH,const_0		;
		ldi	tempreg1,'-'		;sign is negative
libmio_outd_02:	call	libmio_char		;out sign


libmio_outd_03:	mov	tempreg2,ctrl		;format byte
		andi	tempreg2,0x0c		;mask number of digits
		mov	tempreg3,ctrl		;format byte
		andi	tempreg3,0x30		;mask comma position
		clt				;clear nonzero-flag
		ldi	tempreg1,47		;"0"-1 	
libmio_outd_04:	inc	tempreg1			;
		subi	XL,16			;-10000
		sbci	XH,39
		brcc	libmio_outd_04		;no overflow
		subi	XL,240			;+10000
		sbci	XH,216			;
		cpi	tempreg2,0x0c		;5 digits?
		brne	libmio_outd_05		;no		
		rcall	libmio_nc		;out digit 10^4

libmio_outd_05:	ldi	tempreg1,47		;"0"-1 	
libmio_outd_06:	inc	tempreg1			;
		subi	XL,232			;-1000
		sbci	XH,3			;
		brcc	libmio_outd_06		;no overflow
		subi	XL,24			;+1000
		sbci	XH,252			;
		sbrc	tempreg2,3		;skip if < 4 digits
		rcall	libmio_nc		;out digit 10^3
		cpi	tempreg3,0x30		;comma?
		brne	libmio_outd_07		;no
		rcall	libmio_comma		;

libmio_outd_07:	ldi	tempreg1,47		;"0"-1 	
libmio_outd_08:	inc	tempreg1		;+1
		subi	XL,100			;-100
		sbc	XH,const_0		;
		brcc	libmio_outd_08		;no overflow
		subi	XL,156			;+100
		sbci	XH,255			;
		cpi	tempreg2,0x00		;3 digits
		breq	libmio_outd_09
		rcall	libmio_nc		;out digit 10^2
libmio_outd_09:	cpi	tempreg3,0x20		;comma?
		brne	libmio_outd_10		;no
		rcall	libmio_comma		;

libmio_outd_10:	ldi	tempreg1,47		;"0"-1 	
libmio_outd_11:	inc	tempreg1		;+1
		subi	XL,10			;-10
		brcc	libmio_outd_11		;no overflow
		rcall	libmio_nc		;out digit 10^1
		subi	XL,246			;+10
		cpi	tempreg3,0x10		;comma?
		brne	libmio_outd_12		;no
		rcall	libmio_comma		;
		
libmio_outd_12:	ldi	tempreg1,48		;"0"
		add	tempreg1,XL		;LSB digit
		rcall	libmio_char
		pop	XH
		pop	XL
		pop	tempreg3
		pop	tempreg2
		pop	tempreg1
		ret


;-------------------------------------------------------------------------------
; out numeric char (tempreg1)
;-------------------------------------------------------------------------------
libmio_nc:	cpi	tempreg1,48		;is zero
		breq	libmio_nc_1		;yes
		set				;set flag
		rjmp	libmio_char		;send char
		
libmio_nc_1:	sbrc	ctrl,1			;skip if no leading zeroes
libmio_nc_2:	rjmp	libmio_char		;send zero
		brts	libmio_nc_2		;flag is set
		
		sbrs	ctrl,0			;skip if no leading spaces
		ret 
		rjmp	libmio_space		;send space
		ret
.endif

;-------------------------------------------------------------------------------
; Bin -> ASCII (hex) 
; (XL,XH) = number
; ctrl 		= format
; 	Bit 0	1= 16Bit (4 chars)
;	Bit 6	1= Big size
;-------------------------------------------------------------------------------
.ifdef use_libmio_outh
libmio_outh:	push	tempreg1		;save temp register
		sbrs	ctrl,0
		rjmp	libmio_outh1		;only 8
		mov	tempreg1,XH		;
		swap	tempreg1		;high-nibble
		rcall	libmio_oh
		mov	tempreg1,XH		;
		rcall	libmio_oh
libmio_outh1:	mov	tempreg1,XL		;
		swap	tempreg1		;high-nibble
		rcall	libmio_oh
		mov	tempreg1,XL		;
		rcall	libmio_oh
		pop	tempreg1		;restore temp register
		ret

;-------------------------------------------------------------------------------
; out hexadecimal char (tempreg1)
;-------------------------------------------------------------------------------
libmio_oh:	andi	tempreg1,0x0f		;mask nibble
		subi	tempreg1,0xd0		;+30
		cpi	tempreg1,0x3a		;>9
		brcs	libmio_oh_1		;jump, if not
		subi	tempreg1,0xf9		;+6
libmio_oh_1:	rjmp	libmio_char		
.endif
		
;-------------------------------------------------------------------------------
; clear video-RAM
;-------------------------------------------------------------------------------
libmio_clr:	push	XL
		push	XH
		push	YL
		push	YH
		ldi	YH,HIGH(libmio_vram)	;home position
		ldi	YL,LOW(libmio_vram)	;home position
		ldi	XH,0x00
libmio_cls_g01:	st	Y+,XH			;write to RAM
		cpi	YH,HIGH(libmio_vram+1024)		
		brne	libmio_cls_g01		;loop
		cpi	YL,LOW(libmio_vram+1024)		
		brne	libmio_cls_g01		;loop		
		clr	libmio_cur_x		;set X-position
		clr	libmio_cur_y		;set Y-position
		sts	libmio_lpos,const_0
		pop	YH
		pop	YL
		pop	XH
		pop	XL
		ret
		
;-------------------------------------------------------------------------------
; newline and calculate new position
;-------------------------------------------------------------------------------
.ifdef use_libmio_newline
libmio_newline:	push	ZH
		push	ZL
		push	YH
		push	YL
		push	XH
		push	XL
		push	tempreg1
		lds	YL,libmio_font
		clr	libmio_cur_x		;clear X-position
		mov	XH,libmio_cur_y		;get Y-position
		subi	XH,0xf6			;+10
		mov	libmio_cur_y,XH
		cpi	XH,54
		brcc	libmio_newl_02		
libmio_newl_01:	pop	tempreg1
		pop	XL
		pop	XH
		pop	YL
		pop	YH
		pop	ZL
		pop	ZH
		ret
libmio_newl_02:	ldi	ZL,LOW(libmio_vram)
		ldi	ZH,HIGH(libmio_vram)
		ldi	YL,LOW(libmio_vram+160)
		ldi	YH,HIGH(libmio_vram+160)
		ldi	XH,54			;lines
libmio_newl_03:	ldi	XL,16			;bytes per line
libmio_newl_04:	ld	tempreg1,Y+
		st	Z+,tempreg1
		dec	XL
		brne	libmio_newl_04
		dec	XH
		brne	libmio_newl_03		
		ldi	XL,160
libmio_newl_05:	st	Z+,const_0
		dec	XL
		brne	libmio_newl_05
		mov	XH,libmio_cur_y		;get Y-position
		subi	XH,0x0a			;+10
		mov	libmio_cur_y,XH
		rjmp	libmio_newl_01	
.endif

;-------------------------------------------------------------------------------
; output character and calculate new position (tempreg1=char)
;-------------------------------------------------------------------------------
libmio_comma:	ldi	tempreg1,0x2c		;set char to comma
		rjmp	libmio_char		;write
libmio_space:	ldi	tempreg1,32		;set char to space
libmio_char:	push	XL
		push	XH
		lds	XL,libmio_lpos
		inc	XL
		sts	libmio_lpos,XL
		lds	XL,libmio_channel
		cpi	XL,4
		brcs	libmio_chr_1
		pop	XH
		pop	XL
		mov	tempreg2,tempreg1	;copy char
		libi2c_wbyte			;output to i2c
		ret
		
libmio_chr_1:	cpi	XL,0			;screen
		breq	libmio_char2
		cpi	XL,3			;array
		brne	libmio_char0
		mov	XL,tempreg1
		clr	XH
		push	tempreg2
		push	tempreg3
		call	bas_warr		;write to array
		pop	tempreg3
		pop	tempreg2
		pop	XH
		pop	XL
		ret	
		
libmio_char0:	pop	XH
		pop	XL
		lds	XL,libmio_channel
		sbrs	XL,1			;serial
		jmp	libmio_pser		;write serial
libmio_char1:	rjmp	libmio_ppar		;write parallel

libmio_char2:	push	ZL			;save
libmio_gchar:	push	ZH			;restore Z reg
		push	YL
		push	YH
		push	tempreg3
		push	tempreg4
		mov	ZL,tempreg1		;char
		lds	tempreg3,libmio_font	;get font
		sbrc	tempreg3,0		;skip if font 0
		rjmp	libmio_gchar5		;jump to fnt1 routine
		
		ldi	ZH,HIGH(libmio_ctable*2)	;start char ROM
		mov	XL,libmio_cur_x		;get X-position
		mov	XH,libmio_cur_y		;get Y-position
		ldi	YH,10			;number of lines
		
libmio_gchar1:	lpm	YL,Z			;get pixline
		rcall	libmio_plpl
		inc	XH
		mov	XL,libmio_cur_x		;get X-position
		sbrc	ctrl,6			;bigchars
		rcall	libmio_plpl
		sbrc	ctrl,6			;bigchars
		inc	XH
		inc	ZH			;next pixline
		mov	XL,libmio_cur_x		;get X-position
		dec	YH
		brne	libmio_gchar1		
		mov	XL,libmio_cur_x		;get X-position
		subi	XL,0xfa			;+6
		sbrc	ctrl,6
		subi	XL,0xfa			;+6
		mov	libmio_cur_x,XL

libmio_gchar4:	pop	tempreg4	
		pop	tempreg3
		pop	YH
		pop	YL
		pop	ZH		
		pop	ZL
		pop	XH
		pop	XL
		ret

libmio_gchar5:	ldi	ZH,HIGH(libmio_ctable1*2)	;start char ROM
		andi	ZL,0x7f
		mov	XL,ZL
		ldi	ZL,LOW(libmio_ctable1*2)	;start char ROM
		add	ZL,XL
		adc	ZH,const_0				
		lsl	XL
		add	ZL,XL
		adc	ZH,const_0
		
		mov	XL,libmio_cur_x		;get X-position
		mov	XH,libmio_cur_y		;get Y-position
		ldi	YH,6			;number of lines
		
libmio_gchar6:	lpm	YL,Z			;get pixline
		rcall	libmio_plpl5		
		inc	XH
		mov	XL,libmio_cur_x		;get X-position
		sbrc	ctrl,6			;bigchars
		rcall	libmio_plpl5
		sbrc	ctrl,6			;bigchars
		inc	XH
		sbrc	YH,0			;skip if first nibble is used
		adiw	ZL,1
		mov	XL,libmio_cur_x		;get X-position
		dec	YH
		brne	libmio_gchar6		
		mov	XL,libmio_cur_x		;get X-position
		subi	XL,0xfc			;+4
		sbrc	ctrl,6
		subi	XL,0xfc			;+4
		mov	libmio_cur_x,XL
		rjmp	libmio_gchar4

;-------------------------------------------------------------------------------
; plot pixline in graphics mode with font 0, YL=pixline
;-------------------------------------------------------------------------------
libmio_plpl:	lds	tempreg3,libmio_color
		mov	tempreg4,tempreg3
		sbrs	YL,7			;pix 1
		com	tempreg4		
		sts	libmio_color,tempreg4
		rcall	libmio_plot		;output pix
		inc	XL			;next pos
		sbrc	ctrl,6			;no bigchar
		rcall	libmio_plot		;output pix again
		sbrc	ctrl,6			;no bigchar
		inc	XL			;next pos
	
		mov	tempreg4,tempreg3
		sbrs	YL,6			;pix 1
		com	tempreg4		
		sts	libmio_color,tempreg4
		rcall	libmio_plot		;output pix
		inc	XL			;next pos
		sbrc	ctrl,6			;no bigchar
		rcall	libmio_plot		;output pix again
		sbrc	ctrl,6			;no bigchar
		inc	XL			;next pos
		
		mov	tempreg4,tempreg3
		sbrs	YL,5			;pix 1
		com	tempreg4		
		sts	libmio_color,tempreg4
		rcall	libmio_plot		;output pix
		inc	XL			;next pos
		sbrc	ctrl,6			;no bigchar
		rcall	libmio_plot		;output pix again
		sbrc	ctrl,6			;no bigchar
		inc	XL			;next pos

		mov	tempreg4,tempreg3
		sbrs	YL,4			;pix 1
		com	tempreg4		
		sts	libmio_color,tempreg4
		rcall	libmio_plot		;output pix
		inc	XL			;next pos
		sbrc	ctrl,6			;no bigchar
		rcall	libmio_plot		;output pix again
		sbrc	ctrl,6			;no bigchar
		inc	XL			;next pos

		mov	tempreg4,tempreg3
		sbrs	YL,3			;pix 1
		com	tempreg4		
		sts	libmio_color,tempreg4
		rcall	libmio_plot		;output pix
		inc	XL			;next pos
		sbrc	ctrl,6			;no bigchar
		rcall	libmio_plot		;output pix again
		sbrc	ctrl,6			;no bigchar
		inc	XL			;next pos
		
		mov	tempreg4,tempreg3
		sbrs	YL,2			;pix 2
		com	tempreg4		
		sts	libmio_color,tempreg4
		rcall	libmio_plot		;output pix
		inc	XL			;next pos
		sbrc	ctrl,6			;no bigchar
		rcall	libmio_plot		;output pix again
		sbrc	ctrl,6			;no bigchar
		inc	XL			;next pos

		sts	libmio_color,tempreg3
		ret

;-------------------------------------------------------------------------------
; plot pixline in graphics mode with font 1, YL=pixline
;-------------------------------------------------------------------------------
libmio_plpl5:	lds	tempreg3,libmio_color
		sbrs	YH,0
		swap	YL
libmio_plpl5a:	mov	tempreg4,tempreg3
		sbrs	YL,3			;pix 1
		com	tempreg4		
		sts	libmio_color,tempreg4
		rcall	libmio_plot		;output pix
		inc	XL			;next pos
		sbrc	ctrl,6			;no bigchar
		rcall	libmio_plot		;output pix again
		sbrc	ctrl,6			;no bigchar
		inc	XL			;next pos
		
		mov	tempreg4,tempreg3
		sbrs	YL,2			;pix 2
		com	tempreg4		
		sts	libmio_color,tempreg4
		rcall	libmio_plot		;output pix
		inc	XL			;next pos
		sbrc	ctrl,6			;no bigchar
		rcall	libmio_plot		;output pix again
		sbrc	ctrl,6			;no bigchar
		inc	XL			;next pos

		mov	tempreg4,tempreg3
		sbrs	YL,1			;pix 3
		com	tempreg4		
		sts	libmio_color,tempreg4
		rcall	libmio_plot		;output pix
		inc	XL			;next pos
		sbrc	ctrl,6			;no bigchar
		rcall	libmio_plot		;output pix again
		sbrc	ctrl,6			;no bigchar
		inc	XL			;next pos

		mov	tempreg4,tempreg3
		sbrs	YL,0			;pix 4
		com	tempreg4		
		sts	libmio_color,tempreg4
		rcall	libmio_plot		;output pix
		inc	XL			;next pos
		sbrc	ctrl,6			;no bigchar
		rcall	libmio_plot		;output pix again
		sbrc	ctrl,6			;no bigchar
		inc	XL			;next pos

libmio_plpl6:	sts	libmio_color,tempreg3
		ret
			
;-------------------------------------------------------------------------------
; output text from ROM (Z=text,nullterminated)
;-------------------------------------------------------------------------------
.ifdef use_libmio_orom
libmio_orxy:	rcall	libmio_gxy
libmio_orom:	push	tempreg1	
libmio_orom_1:	lpm	tempreg1,Z+		;get char
		cpi	tempreg1,0		;end?
		brne	libmio_orom_2		;no
		pop	tempreg1
		ret				;the end
libmio_orom_2:	rcall	libmio_char		;write char to screen
		rjmp	libmio_orom_1		;next char	
.endif

;-------------------------------------------------------------------------------
; output text from ROM (following y,x,nullterminated text)
;-------------------------------------------------------------------------------
.ifdef use_libmio_othis
libmio_othis:	pop	ZH			;get return address
		pop	ZL
		lsl	ZL			;*2 so we use bytes
		rol	ZH
		push	XL
		push	XH
		lpm	XH,Z+			;get first value (255/Y)
		cpi	XH,0xff			;no position
		breq	libmio_othis_0
		lpm	XL,Z+			;get second value
		rcall	libmio_gxy		;set position
libmio_othis_0:	pop	XH
		pop	XL
		push	tempreg1		;save reg
libmio_othis_1:	lpm	tempreg1,Z+		;get char
		cpi	tempreg1,0		;end?
		brne	libmio_othis_2		;no
		pop	tempreg1		;restore reg
		sbiw	ZL,1			;point to zero
		lsr	ZH			;/2
		ror	ZL
		adiw	ZL,1			;point to next instruction
		ijmp
libmio_othis_2:	rcall	libmio_char		;write char to screen
		rjmp	libmio_othis_1		;next char	
.endif

;-------------------------------------------------------------------------------
; output text from RAM (Z=text,nullterminated)
;-------------------------------------------------------------------------------
.ifdef use_libmio_oram
libmio_oram:	push	tempreg1
libmio_oram_1:	ld	tempreg1,Z+		;get char
		cpi	tempreg1,0		;end?
		brne	libmio_oram_2		;no
		pop	tempreg1
		ret				;the end
libmio_oram_2:	call	libmio_char		;write char to screen
		rjmp	libmio_oram_1		;next char	
.endif

;-------------------------------------------------------------------------------
; output text (multiple char) ZL=number
;-------------------------------------------------------------------------------
.ifdef use_libmio_mtext
libmio_mtext:	call	libmio_char		;write char to screen
		dec	ZL
		brne	libmio_mtext	
		ret
.endif

;-------------------------------------------------------------------------------
; set cursor to position XL:XH
;-------------------------------------------------------------------------------
.ifdef use_libmio_gotoxy
libmio_gxy:	mov	libmio_cur_x,XL		;save X-position
		mov	libmio_cur_y,XH		;save Y-position
		ret
.endif

;-------------------------------------------------------------------------------
; plot  XL=X XH=Y  
;-------------------------------------------------------------------------------
.ifdef use_libmio_plot
libmio_plot:	push	YL
		push	YH
		cpi	XL,0x80
		brcc	libmio_nplot		;yes
		cpi	XH,0x40			;greater?
		brcc	libmio_nplot		;yes
		rjmp	libmio_plot0		;OK, we can plot
libmio_nplot:	pop	YH			;restore registers
		pop	YL
		ret		
		
libmio_plot0:	push	XL			;save coords
		push	XH
		push	tempreg1
		push	tempreg2
libmio_plot1_0:	mov	YL,XL			;X
		andi	YL,0x07			;only bits
		inc	YL			;+1
		ldi	tempreg1,0x00		;bitmask
		sec				;set carry
libmio_plot1_1:	rol	tempreg1		;shift in bit
		dec	YL
		brne	libmio_plot1_1
;bit mask is in tempreg1
		lsr	XL			;X/8
		lsr	XL
		lsr	XL
		ldi	YL,16		
		mul	XH,YL
		ldi	YH,HIGH(libmio_vram)	;home position
		ldi	YL,LOW(libmio_vram)	;home position
		add	YL,XL
		adc	YH,const_0
		add	YL,r0
		adc	YH,r1
;address is now in Y	    	
		lds	XL,libmio_color		;active color
		andi	XL,0x01			;we check only bit 0
		brne	libmio_plot1_2		;set
		com	tempreg1
		ld	XL,Y
		and	XL,tempreg1
		st	Y,XL
libmio_plot_os:	pop	tempreg2
		pop	tempreg1
		pop	XH
		pop	XL
		pop	YH
		pop	YL
		ret		
				
libmio_plot1_2:	ld	XL,Y
		or	XL,tempreg1
		st	Y,XL		
		rjmp	libmio_plot_os

;-------------------------------------------------------------------------------
; correct char coordinates from YL,YH to ZL,ZH
;-------------------------------------------------------------------------------
libmio_corrc:	cpi	YL,libmio_cols
		brcs	libmio_corrc1
		ldi	YL,libmio_cols-1
libmio_corrc1:	cpi	ZL,libmio_cols
		brcs	libmio_corrc2
		ldi	ZL,libmio_cols-1
libmio_corrc2:	cpi	YH,libmio_rows
		brcs	libmio_corrc3
		ldi	YH,libmio_rows-1
libmio_corrc3:	cpi	ZH,libmio_rows
		brcs	libmio_corrc4
		ldi	ZH,libmio_rows-1
libmio_corrc4:	cp	ZL,YL
		brcc	libmio_corrc5
		mov	r0,ZL
		mov	ZL,YL
		mov	YL,r0
libmio_corrc5:	cp	ZH,YH
		brcc	libmio_corrc6
		mov	r0,ZH
		mov	ZH,YH
		mov	YH,r0
libmio_corrc6:	ret		

;-------------------------------------------------------------------------------
; correct plot coordinates from YL,YH to ZL,ZH
;-------------------------------------------------------------------------------
libmio_corrp:	cp	ZL,YL
		brcc	libmio_corrp1
		mov	r0,ZL
		mov	ZL,YL
		mov	YL,r0
libmio_corrp1:	cp	ZH,YH
		brcc	libmio_corrp2
		mov	r0,ZH
		mov	ZH,YH
		mov	YH,r0
libmio_corrp2:	ret		

;-------------------------------------------------------------------------------
; box YL,YH to ZL,ZH
;-------------------------------------------------------------------------------
.ifdef use_libmio_box
libmio_box:	push	XL
		push	XH
		push	YL
		push	YH
		push	ZL
		push	ZH
		push	tempreg2
		rcall	libmio_corrp
libmio_box_2:	mov	XL,YL
		mov	XH,YH
libmio_box_3:	rcall	libmio_plot
		inc	XL
		cp	XL,ZL
		brne	libmio_box_3
libmio_box_4:	rcall	libmio_plot
		inc	XH
		cp	ZH,XH
		brne	libmio_box_4
libmio_box_5:	rcall	libmio_plot
		dec	XL
		cp	XL,YL
		brne	libmio_box_5
libmio_box_6:	rcall	libmio_plot
		dec	XH
		cp	XH,YH
		brne	libmio_box_6
		pop	tempreg2
		pop	ZH
		pop	ZL
		pop	YH
		pop	YL
		pop	XH
		pop	XL
		ret				
.endif				

;-------------------------------------------------------------------------------
; fbox YL,YH to ZL,ZH
;-------------------------------------------------------------------------------
.ifdef use_libmio_fbox
libmio_fbox:	push	XL
		push	XH
		push	YL
		push	YH
		push	ZL
		push	ZH
		push	tempreg2
		rcall	libmio_corrp
libmio_fbox_2:	mov	XH,YH
libmio_fbox_3:	mov	XL,YL
libmio_fbox_4:	rcall	libmio_plot
		cp	XL,ZL
		breq	libmio_fbox_5
		inc	XL
		rjmp	libmio_fbox_4
libmio_fbox_5:	rcall	libmio_plot
		cp	ZH,XH
		breq	libmio_fbox_6
		inc	XH
		rjmp	libmio_fbox_3
.endif
		
libmio_fbox_6:	pop	tempreg2
libmio_fbox_7:	pop	ZH
		pop	ZL
		pop	YH
		pop	YL
		pop	XH
		pop	XL
		ret				


;-------------------------------------------------------------------------------
; draw line from YL,YH to ZL,ZH
; this algorithm is a little bit quick and dirty, but it works...
;-------------------------------------------------------------------------------
.ifdef use_libmio_draw
libmio_draw:	push	r16
		push	r17
		push	r18
		push	r19
		push	XL
		push	XH
		push	YL
		push	YH		
		movw	XL,YL			;copy start pointer
		ldi	r16,0xff		;set sgn(dx) negative
		sub	YL,ZL			;x1-x2
		brcc	libmio_draw01		;branch if x1<x2
		neg	r16			;invert sgn(dx)
		neg	YL			;invert dx
libmio_draw01:	ldi	r17,0xff		;set sgn(dy) negative
		sub	YH,ZH			;-dy
		brcc	libmio_draw02		;branch if y1<y2
		neg	r17			;invert sgn(dy)
		neg	YH			;invert dy
libmio_draw02:	cp	YL,YH			;|dx|-|dy|
		brcs	libmio_draw6		;branch if |dy|>|dx|
		rjmp	libmio_draw3

		

;|dx|>|dy|		
libmio_draw3:	mov	r18,YL			;|dx|
		lsr	r18			;/2
libmio_draw4:	rcall	libmio_plot		;plot pixel
		cp	XL,ZL			;the endpoint?
		breq	libmio_draw_e		;yes
		add	XL,r16			;X-step
		sub	r18,YH			;-|dy|
		brcc	libmio_draw4		;no overflow
		add	XH,r17			;y-step
		add	r18,YL
		rjmp	libmio_draw4			

;|dy|>|dx|
libmio_draw6:	mov	r18,YH			;|dy|
		lsr	r18			;/2
libmio_draw7:	rcall	libmio_plot		;plot pixel
		cp	XH,ZH			;the endpoint?
		breq	libmio_draw_e
		add	XH,r17			;y-step
		sub	r18,YL
		brcc	libmio_draw7			
		add	XL,r16			;x-step
		add	r18,YH
		rjmp	libmio_draw7			

libmio_draw_e:	pop	YH
		pop	YL
		pop	XH
		pop	XL
		pop	r19
		pop	r18
		pop	r17
		pop	r16
		ret		


.endif


;-------------------------------------------------------------------------------
; wait for the end of visible frame
;-------------------------------------------------------------------------------
.ifdef use_libmio_sync
libmio_sync:	push	XL
libmio_sync_1:	mov	XL,vline_l
		cpi	XL,192
		breq	libmio_sync_1
libmio_sync_2:	mov	XL,vline_l
		cpi	XL,192
		brne	libmio_sync_2
		pop	XL
		ret		    
.endif


;-------------------------------------------------------------------------------
; play a note (XL=note 0..63 0xff=>noise)
;-------------------------------------------------------------------------------
.ifdef use_libmio_note
libmio_note:	push	ZL
		push	ZH
		push	XL
		sts	libmio_note_val,XL	;write note value
		andi	XL,0x3f			;64 notes
		lsl	XL
		ldi	ZH,HIGH(libmio_ntable*2)	;1 start envtable high
		ldi	ZL,LOW(libmio_ntable*2)	;1 start envtable low
		add	ZL,XL			;offsettable
		adc	ZH,const_0		;
		lpm	XL,Z+			;get low offset
		sts	libmio_woff_l,XL	;write
		lpm	XL,Z+			;get high offset
		sts	libmio_woff_h,XL	;write
		ldi	ZL,15			;duration
		lds	XL,libmio_sysconf
		sbrc	XL,3
		sts	libmio_env,ZL		;start
		sts	libmio_wpos_l,const_0
		sts	libmio_wpos_h,const_0
		pop	XL 
		pop	ZH
		pop	ZL
		ret		    
.endif

;------------------------------------------------------------------------------
;wait for a key
;------------------------------------------------------------------------------
.ifdef use_libmio_waitkey
libmio_waitkey:	sts	libmio_lastkey,const_0
libmio_wkey1:	lds	tempreg1,libmio_lastkey
		cpi	tempreg1,0
		breq	libmio_wkey1
		rcall	libmio_sync
		ret
.endif				

;------------------------------------------------------------------------------
;wait for no key
;------------------------------------------------------------------------------
libmio_nokey:	lds	tempreg1,libmio_keycode
		cpi	tempreg1,0
		brne	libmio_nokey
		rcall	libmio_sync
		lds	tempreg1,libmio_keycode
		cpi	tempreg1,0
		brne	libmio_nokey
		rcall	libmio_sync
		lds	tempreg1,libmio_keycode
		cpi	tempreg1,0
		brne	libmio_nokey
		ret

;------------------------------------------------------------------------------
;wait for no key
;------------------------------------------------------------------------------
libmio_nokeys:	lds	tempreg1,libmio_keycode
		cpi	tempreg1,0
		brne	libmio_nokeys
		rcall	libmio_sync
		ret
		
;------------------------------------------------------------------------------
;put tempreg1 to printer output
;------------------------------------------------------------------------------
libmio_ppar:	ret				;thats all


;------------------------------------------------------------------------------
;put tempreg1 to serial output
;------------------------------------------------------------------------------
libmio_pser:	push	tempreg1
libmio_pser1:	lds	tempreg1,UCSR0A
		sbrs	tempreg1,UDRE0
		rjmp	libmio_pser1
		pop	tempreg1
		sts	UDR0,tempreg1
		ret				;thats all

;------------------------------------------------------------------------------
;put null-terminated flashtext to serial port
;------------------------------------------------------------------------------
libmio_fsert:	lpm	tempreg1,Z+
		cpi	tempreg1,0x00
		breq	libmio_fsert1
		rcall	libmio_pser
		rjmp	libmio_fsert
libmio_fsert1:	ret		

;------------------------------------------------------------------------------
;get tempreg1 from serial input
;------------------------------------------------------------------------------
libmio_gser:	lds	tempreg1,UCSR0A
		sbrs	tempreg1,RXC0
		rjmp	libmio_gser
		lds	tempreg1,UDR0
		ret				;thats all

;------------------------------------------------------------------------------
;get tempreg1 from serial input with break
;------------------------------------------------------------------------------
libmio_gserb:	lds	tempreg1,UCSR0A
		sbrc	tempreg1,RXC0
		rjmp	libmio_gserb_3
libmio_gserb_2:	lds	tempreg1,libmio_keycode	;get keycode
		cpi	tempreg1,0x00		;key pressed?
		breq	libmio_gserb		;no key is pressed
		ldi	tempreg1,0x1b		;break code
		ret				;thats all
			
libmio_gserb_3:	lds	tempreg1,UDR0
		ret				;thats all

;-------------------------------------------------------------------------------
; set config for I2C and SPI
;-------------------------------------------------------------------------------
libmio_setser:	push	YH
		push	YL
		push	XH
		push	XL
		rjmp	libmio_setserx

libmio_setconf:	push	YH
		push	YL
		push	XH
		push	XL
		ldi	XL,0x5c		;f/4
		lds	XH,libmio_sysconf
		sbrc	XH,1
		ldi	XL,0x5f		;f/128
		out	SPCR0,XL
		ldi	XL,92		;speed
		sts	TWBR,XL		
		ldi	XL,0x00		;/1
		sbrs	XH,2
		ldi	XL,0x01		;/4
		sts	TWSR,XL
libmio_setserx:	andi	XH,0xe0		;mask bits
		ldi	YL,LOW(2082)	;UBBR for 1200
		ldi	YH,HIGH(2082)
		cpi	XH,0x20		;2400
		brne	libmio_setcnf1
		ldi	YL,LOW(1041)	;UBBR for 2400
		ldi	YH,HIGH(1041)
libmio_setcnf1:	cpi	XH,0x40		;4800
		brne	libmio_setcnf2
		ldi	YL,LOW(520)	;UBBR for 4800
		ldi	YH,HIGH(520)
libmio_setcnf2:	cpi	XH,0x60		;9600
		brne	libmio_setcnf3
		ldi	YL,LOW(259)	;UBBR for 9600
		ldi	YH,HIGH(259)
libmio_setcnf3:	cpi	XH,0x80		;19200
		brne	libmio_setcnf4
		ldi	YL,LOW(129)	;UBBR for 19200
		ldi	YH,HIGH(129)
libmio_setcnf4:	cpi	XH,0xa0		;31250
		brne	libmio_setcnf5
		ldi	YL,LOW(80)	;UBBR for 31250
		ldi	YH,HIGH(80)
libmio_setcnf5:	cpi	XH,0xc0		;38400
		brne	libmio_setcnf6
		ldi	YL,LOW(64)	;UBBR for 38400
		ldi	YH,HIGH(64)
libmio_setcnf6:	cpi	XH,0xe0		;57600
		brne	libmio_setcnfe
		ldi	YL,LOW(42)	;UBBR for 57600
		ldi	YH,HIGH(42)
libmio_setcnfe:	sts	UBRR0H,YH
		sts	UBRR0L,YL
		pop	XL
		pop	XH
		pop	YL
		pop	YH			
		ret		
				
;-------------------------------------------------------------------------------
; pseudo-random generator (external call)
;-------------------------------------------------------------------------------
libmio_psrg:	push	ZH
		push	ZL
		push	r16
		lds	ZL,libmio_rand1
		lds	XL,libmio_rand2
		lds	XH,libmio_rand3
		lds	ZH,libmio_rand4
		clr	r16
		sbrc	ZL,2
		com	r16
		sbrc	ZL,5
		com	r16
		sbrc	XL,4
		com	r16
		sbrc	ZH,7
		com	r16
		rol	r16
		rol	ZL
		rol	XL
		rol	XH
		rol	ZH
		sts	libmio_rand1,ZL
		sts	libmio_rand2,XL
		sts	libmio_rand3,XH
		sts	libmio_rand4,ZH
		pop	r16
		pop	ZL
		pop	ZH
		ret
		

.include 	"libmio/bcopy.asm"
.include 	"libmio/circle.asm"
.include 	"libmio/gui.asm"
				
		

		