;################################################################################
;# AVR-ChipBasic2m v0.34							#
;# handheld computer with ATmega644						#
;# copyright (c) 2007-2008 Joerg Wolfram (joerg@jcwolfram.de)			#
;#										#
;#										#
;# This program is free software; you can redistribute it and/or		#
;# modify it under the terms of the GNU General Public License as 		#
;# published by the Free Software Foundation; either version 2 of		#
;# the License, or (at your option) any later version.				#
;#										#
;# This library is distributed in the hope that it will be useful,		#
;# but WITHOUT ANY WARRANTY; without even the implied warranty of		#
;# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the GNU		#
;# Lesser General Public License for more details.				#
;#										#
;# You should have received a copy of the GNU Lesser General Public		#
;# License along with this library; if not, write to the			#
;# Free Software Foundation, Inc., 59 Temple Place - Suite 330,			#
;# Boston, MA 02111-1307, USA.							#
;#										#
;################################################################################

.CSEG
.include	"m644def.inc"
.include	"libdef.asm"			;register definitions
.def		vline_l = r8
.def		vline_h = r9

.equ		libmio_vram	= 0x100		;1024 bytes of video RAM (pixel)
.equ		libmio_ram	= 0x500		;80 bytes libmio system RAM
.equ		bas_partab	= 0x550		;16 bytes basic parameter (used by BASIC programs)
.equ		partab		= 0x550		;16 bytes parameter table (used by others)
.equ		libdfl_buffer	= 0x560		;256 bytes file buffer
.equ		libi2c_buffer	= 0x560		;libi2c uses the file buffer
.equ		libdfl_ram	= 0x670		;16 Bytes libdfl system RAM


;for BASIC programs
.equ		bas_ram		= 0x680		;16 bytes basic system RAM
.equ		bas_inbuf	= 0x690		;40 bytes input buffer
.equ		bas_linebuf	= 0x6c0		;40 bytes line buffer
.equ		varspace	= 0x700		;52 bytes basic variables
.equ		expar_ros	= 0x75a		;38 bytes calculator stack
.equ		bas_stacks	= 0x7ec		;70 bytes basic stacks
.equ		bcopy_buffer	= 0x840		;bcopy buffer
.equ		bas_array	= 0xd80		;768 Bytes Array

;menu ramspace
.equ		libmio_mram	= 0x680		;128 bytes menu buffer
.equ		libmio_tram	= 0x700		;64 bytes keyboard ram

;mini la/curve
.equ		rdata_ram	= 0x740		;miniLA data
.equ		rdata_head	= 0x750		;miniLA head buffer 
.equ		rdata_buf	= 0x780		;miniLA record buffer to 0xf7f

.equ		timer_ram	= 0xf80		;16 bytes timer RAM
.equ		time_ram	= 0xf80		;16 bytes time meas RAM
.equ		count_ram	= 0xf90		;16 bytes counter RAM

.equ		stack		= 0x10ff	;stack


.include	"modules/basic/definitions.asm"
.include	"libmio/definitions.asm"
.include	"libeep/definitions.asm"
.include	"libi2c/definitions.asm"
.include	"macros.inc"

.set		bas_lines=95
.set		bas_stackdepth = 14
.set		bas_maxprog = 8

.org 0x0000
		rjmp	start
.org 0x0004
		.db 0x56,0x78
.org OC1Baddr	
		libmio_vint			;video interrupt
			
ident:		.db "AVR-Handheld (c) 2008 Joerg Wolfram",0
.org 0x0038
.include	"modules/api.asm"		;api for user programs
start:		cli
		ldi	XL,0x00
		mov	const_0,XL
		ldi	XL,0x01
		mov	const_1,XL
		ldi	XL,HIGH(stack)
		out	SPH,XL
		ldi	XL,LOW(stack)
		out	SPL,XL
wloop1:		ldi	XH,0x00
wloop2:		ldi	XL,0x00		
wloop3:		dec	XL
		brne	wloop3
		dec	XH
		brne	wloop2

		call	mem_getconf
		libmio_init
		libi2c_init
		call	mem_getconf
		libmio_sysconf
		libmio_clrscr
		sts	libmio_seqspeed,const_0
wloop4:		clr	ereg
		sei
				
loop:		jmp	intro_main		;main menu
		rjmp	start			;run again

		libdfl_gettype			;enable libdfl
		libgps_dummy			;enable libgps
		libeep_read			;enable libeep
		
;-----------------------------------------------------------------------------
;common
;-----------------------------------------------------------------------------				
.include	"modules/intro.asm"		;intro screen
.include	"modules/menu.asm"		;main menu
libmio_icons:
.include	"libmio/itable1.inc"		;icon table
.include	"lang/de/text.inc"		;german translation

;-----------------------------------------------------------------------------
;file
;-----------------------------------------------------------------------------
.include	"modules/file/file.asm"		;file functions
.include	"modules/file/filesys.asm"		;filesystem functions

;-----------------------------------------------------------------------------
;timer / watch
;-----------------------------------------------------------------------------
.include	"modules/timer/timer.asm"	;timer_menu
.include	"modules/timer/set.asm"		;RTC set
.include	"modules/timer/alarm.asm"	;alarm
.include	"modules/timer/stwatch.asm"	;stop-watch

;-----------------------------------------------------------------------------
;configuration
;-----------------------------------------------------------------------------
.include	"modules/config.asm"		;config menu

;-----------------------------------------------------------------------------
;basic / user programs
;-----------------------------------------------------------------------------
.include	"modules/basic/user.asm"	;run user prog
.include	"modules/basic/exparser.asm"	;BASIC expression parser
.include	"modules/basic/interpreter.asm"	;BASIC interpreter
.include	"modules/basic/spi.asm"		;SPI-routines for BASIC
.include	"modules/basic/array.asm"	;Array routines for BASIC
bas_sintab:
.include	"modules/basic/sintable.inc"

;-----------------------------------------------------------------------------
;measurement
;-----------------------------------------------------------------------------
.include	"modules/meas/thermo.asm"	;thermometer
.include	"modules/meas/volt.asm"		;voltmeter
.include	"modules/meas/meas.asm"		;measurement menu
.include	"modules/meas/minla.asm"	;mini LA
.include	"modules/meas/mincu.asm"	;mini Curve
.include	"modules/meas/time.asm"		;time measurement
.include	"modules/meas/count.asm"	;pulse counter
.include	"modules/meas/snif.asm"		;serial sniffer

;-----------------------------------------------------------------------------
;games
;-----------------------------------------------------------------------------
.include	"modules/game/games.asm"	;games menu
.include	"modules/game/bout.asm"		;breakout game
.include	"modules/game/puzzle.asm"	;puzzle game

;-----------------------------------------------------------------------------
;GPS
;-----------------------------------------------------------------------------
.include	"modules/gps/gps.asm"		;gps-tool
.include	"modules/gps/gps2.asm"		;gps2-tool
.include	"modules/gps/gps3.asm"		;gps3-tool
gps_atab:
.include	"modules/gps/cstab.inc"		;yx-table for angle view
gps2_rtab:
.include	"modules/gps/gps_rtab.inc"		;atan2 table
gps2_costab:
.include	"modules/gps/gps_costab.inc"		;cosinus table


;-----------------------------------------------------------------------------
;library stuff
;-----------------------------------------------------------------------------
.include	"libmio/bimode1.asm"		;bimode routines normal orientation
.include	"libmio/bimode2.asm"		;bimode routines rotated orientation

.include	"libs.inc"

libmio_ctable1:
.include	"libmio/ctable_f1.inc"

;screenshot area
.org	0x5b00		
scrs_loc:	.db 0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55
		.db 0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa
		.db 0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55
		.db 0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa,0xaa

.org	0x5d00
libmio_costab:	
.include	"libmio/costab.inc"

.org	0x5d80
.include	"libmio/soundtables.inc"

.org	0x6000
bas_programs:

;program 1 6000-65ff
.org	0x6000
		.db "Hallo Welt  B "
.org	0x6010	
		.db 0x80,0x22,"Hallo Welt!",0x22

;program 2 6600-6bff
.org	0x6600
		.db "              "

;program 3 6c00-71ff
.org	0x6c00
		.db "              "

;program 4 7200-77ff
.org	0x7200
		.db "              "

;---------------------------------------------------------------------------------
;start of nrww-section
;---------------------------------------------------------------------------------
.org	0x7800
.include	"modules/memory.asm"		;flash programming
.include	"modules/file/xm.asm"		;XMODEM routines

.org	0x7b00
libmio_ctable:
.include	"libmio/ctable.inc"		;the main chartable
