;################################################################################
;#										#
;# avr-chipbasic2M - mobile single chip basic computer with ATmega644		#
;# breakout game								#
;# copyright (c) 2008 Joerg Wolfram (joerg@jcwolfram.de)			#
;#										#
;#										#
;# This program is free software; you can redistribute it and/or		#
;# modify it under the terms of the GNU General Public License			#
;# as published by the Free Software Foundation; either version 2		#
;# of the License, or (at your option) any later version.			#
;#										#
;# This program is distributed in the hope that it will be useful,		#
;# but WITHOUT ANY WARRANTY; without even the implied warranty of		#
;# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the GNU		#
;# General Public License for more details.					#
;#										#
;# You should have received a copy of the GNU General Public			#
;# License along with this library; if not, write to the			#
;# Free Software Foundation, Inc., 59 Temple Place - Suite 330,			#
;# Boston, MA 02111-1307, USA.							#
;#										#
;################################################################################

;------------------------------------------------------------------------------
;main screen mask
;------------------------------------------------------------------------------
bout_main:	sts	bas_array+150,const_0		;we start with first level
		mov	tempreg6,const_0		;no sound
		ldi	XL,4				;lifes
		sts	bas_array+151,XL		;set this
		sts	bas_array+148,const_0		;points L
		sts	bas_array+149,const_0		;points H
		ldi	ZL,LOW(libmio_icons*2+530)
		ldi	ZH,HIGH(libmio_icons*2+530)
		libmio_appframe
				
;entry for next level		
bout_main_nl:	rcall	bout_drawfield			;draw playfield
		libmio_nokey				;wait for no key
		ldi	XL,52				;ball X
		sts	bas_array+140,XL
		sts	bas_array+144,XL		;old X
		sts	bas_array+146,XL		;my X
			
		ldi	XL,55				;ball Y
		sts	bas_array+141,XL
		sts	bas_array+145,XL		;old Y		
		sts	bas_array+147,XL		;my old X

		sts	bas_array+143,const_0		;reset counter
		
bout_main_0:	ldi	XL,56				;ball Y
		sts	bas_array+141,XL
		sts	bas_array+145,XL		;old Y
		ldi	XL,2				;start direction
		sts	bas_array+142,XL				
		rcall	bout_show			;show status
		rcall	bout_rball			;kill old ball
		rcall	bout_mfig			;move figure	
		rcall	bout_dfig			;draw figure	
		lds	r16,bas_array+146		;my X
		subi	r16,0xfc			;+4
		sts	bas_array+140,r16	
		lds	XL,bas_array+140		;new X
		lds	XH,bas_array+141		;new Y		
		rcall	bout_dball			;draw ball at new position
		libmio_sync				;wait
		libmio_sync
		lds	tempreg1,libmio_keycode		;key pressed
		cpi	tempreg1,0xed			;ESC
		breq	bout_main_1a			;end
		cpi	tempreg1,0xea			;ENTER
		brne	bout_main_0			;not				
				
		
bout_main_1:	lds	YL,bas_array+143		;increment counter
		inc	YL
		sts	bas_array+143,YL
		
		lds	tempreg1,libmio_keycode		;key pressed
		cpi	tempreg1,0xed			;ESC
		brne	bout_main_11			;not
bout_main_1a:	ret					;end program
bout_main_11:	cpi	tempreg1,0xe4			;UP
		brne	bout_main_2			;not
		inc	tempreg6			;toggle sound flag
		libmio_nokey

bout_main_2:
		rcall	bout_mfig			;move figure	
		rcall	bout_dfig			;draw figure
		rcall	bout_addp			;add points
		cpi	tempreg4,0			;all done
		breq	bout_main_3
		rcall	bout_show		
		libmio_sync				;wait

		rcall	bout_mfig			;move figure	
		rcall	bout_dfig			;draw figure
		libmio_sync				;wait
				
		
		rcall	bout_mfig			;move figure	
		rcall	bout_dfig			;draw figure
		rcall	bout_rball			;remove old ball

		rcall	bout_mball			;calc new ball position
		rcall	bout_cball			;check for moving
		rcall	bout_cdir			;change direction if brick is touched
		
		rcall	bout_mball			;calc new ball position
		rcall	bout_cfig			;check if figure collision
				
		rcall	bout_cborder			;check borders
		
		lds	YL,bas_array+141		;ball Y
		cpi	YL,60				;we not have got it
		breq	bout_main_lo			;lost one live
		
		rcall	bout_mball			;calc new ball position
		rcall	bout_dball			;draw ball at new position
							;wait
		libmio_sync		
		rjmp	bout_main_1

;level is done		
bout_main_3:	lds	XL,bas_array+150		;level
		inc	XL
		andi	XL,3				;only 4 levels available
		sts	bas_array+150,XL		;store level
		rjmp	bout_main_nl			;start with next level		

;lost one life
bout_main_lo:	ldi	XL,0xff
		sbrc	tempreg6,0
		libmio_note
		lds	XL,bas_array+151		;lives
		subi	XL,1
		brcs	bout_main_go			;game over
		sts	bas_array+151,XL		;new lives
		sts	libmio_color,const_0
		ldi	YL,15
		ldi	YH,50
		ldi	ZL,111
		ldi	ZH,62
		libmio_fbox
		sts	libmio_color,const_1
		ldi	XL,52
		sts	bas_array+146,XL		;my X
		inc	XL
		sts	bas_array+147,XL		;my X		
		rjmp	bout_main_0			;ok, we can continue				
					
;game over
bout_main_go:	sts	libmio_font,const_0				
		ldi	ctrl,0x00			;big
		libmio_thistext
		.db	42,33,"Game Over!",0,0	
bout_main_go1:	lds	tempreg1,libmio_keycode		;key pressed
		cpi	tempreg1,0xed			;ESC
		brne	bout_main_go1			;not
		ret					;end program			

;------------------------------------------------------------------------------
;add points and kill bit 7 in field array
;------------------------------------------------------------------------------	
bout_addp:	clr	tempreg4			;counter
		ldi	YL,LOW(bas_array)
		ldi	YH,HIGH(bas_array)
		lds	ZL,bas_array+148		;points L		
		lds	ZH,bas_array+149		;points H
		ldi	XH,96				;to do
bout_addp_1:	ld	XL,Y				;get value
		sbrc	XL,7
		adiw	ZL,1
		andi	XL,0x7f				;kill flag
		st	Y+,XL
		cpi	XL,0x00
		breq	bout_addp_2
		inc	tempreg4			;bricks+1		
bout_addp_2:	dec	XH
		brne	bout_addp_1
		sts	bas_array+148,ZL		;points L
		sts	bas_array+149,ZH		;points H
		ret		

;------------------------------------------------------------------------------
;show data
;------------------------------------------------------------------------------			
bout_show:	sts	libmio_color,const_0
		sts	libmio_font,const_1
		ldi	XL,12
		ldi	XH,2
		libmio_gotoxy
		ldi	XL,0
		lds	XH,bas_array+151
		ldi	tempreg1,'-'
bout_show_1:	cp	XL,XH
		brne	bout_show_2
		ldi	tempreg1,' '
bout_show_2:	libmio_outchar		
		inc	XL
		cpi	XL,4
		brne	bout_show_1
		ldi	XL,34
		ldi	XH,2
		ldi	ZL,LOW(bout_text_1*2)	;text pointer
		ldi	ZH,HIGH(bout_text_1*2)
		libmio_xztext				
		lds	tempreg1,bas_array+150
		subi	tempreg1,0xcf
		libmio_outchar
		ldi	XL,68
		ldi	XH,2
		ldi	ZL,LOW(bout_text_2*2)	;text pointer
		ldi	ZH,HIGH(bout_text_2*2)
		libmio_xztext			
		lds	XL,bas_array+148		;points L		
		lds	XH,bas_array+149		;points H		
		ldi	ctrl,0x0e			;full
		libmio_outdez
		sts	libmio_color,const_1
		ret
	
;------------------------------------------------------------------------------
;check for figure collision (use temporary coords)
;------------------------------------------------------------------------------				
bout_cfig:	cpi	XH,58				;row
		breq	bout_cfig_1			;yes
bout_cfig_0:	ret
		
bout_cfig_1:	lds	YH,bas_array+146		;my X
		lds	YL,bas_array+142		;direction
		andi	YL,0x03				;we can only go up			
		subi	YH,0x80				;+128
		sub	YH,XL				;differnce
		
		cpi	YH,130				;out left 
		brcc	bout_cfig_0	
		
		cpi	YH,119				;out right
		brcs	bout_cfig_0
		
		cpi	YH,129				;end of left edge
		brcs	bout_cfig_2
		dec	YL
		sbrc	YL,7
		clr	YL
		rjmp	bout_cfig_4

bout_cfig_2:	cpi	YH,121				;end of right edge
		brcc	bout_cfig_4
		inc	YL
		sbrc	YL,2
		ldi	YL,3

bout_cfig_4:	push	XL
		ldi	XL,18
		sbrc	tempreg6,0
		libmio_note
		pop	XL
		sts	bas_array+142,YL		;store dir		
		ret					;end
								
;------------------------------------------------------------------------------
;calc new direction from tempreg2
;------------------------------------------------------------------------------		
bout_cdir:	cpi	tempreg2,0			;no collision
		brne	bout_cdir_01
		ret
		
bout_cdir_d:	rcall	bout_nbrick
		rjmp	bout_rdia
				
bout_cdir_h:	rcall	bout_nbrick
		rjmp	bout_rhor
		
bout_cdir_v:	rcall	bout_nbrick
		rjmp	bout_rver		

;only one pix is affected
bout_cdir_01:	cpi	tempreg2,0x01			;upl
		breq	bout_cdir_d
		cpi	tempreg2,0x04			;upr
		breq	bout_cdir_d
		cpi	tempreg2,0x10			;downr
		breq	bout_cdir_d
		cpi	tempreg2,0x40			;downl
		breq	bout_cdir_d
;hor reflect
bout_cdir_02:	mov	tempreg1,tempreg2
		andi	tempreg1,0xf8			;all other	
		breq	bout_cdir_h			;only upper
		mov	tempreg1,tempreg2
		andi	tempreg1,0x8f			;all other
		breq	bout_cdir_h			;only lower		
		
;vert reflect
bout_cdir_03:	mov	tempreg1,tempreg2
		andi	tempreg1,0x3e			;all other
		breq	bout_cdir_v			;only left
		mov	tempreg1,tempreg2
		andi	tempreg1,0xe3			;all other
		breq	bout_cdir_v			;only right			

;2 edges -> diagonal		
		rjmp	bout_cdir_d			;move diag				

		
;------------------------------------------------------------------------------
;
;------------------------------------------------------------------------------		
bout_cborder:	lds	XL,bas_array+140		;X
		lds	XH,bas_array+141		;Y
		lds	YL,bas_array+142		;ball direction		
		cpi	XL,16				;left
		brne	bout_cborder_1			
;left border		
		sbrs	YL,1				;skif if we move right
		rcall	bout_nrver			;reflect
		rjmp	bout_cborder_2
		
bout_cborder_1:	cpi	XL,110				;right
		brne	bout_cborder_2			
;right border		
		sbrc	YL,1				;skif if we move left
		rcall	bout_nrver			;reflect

bout_cborder_2:	cpi	XH,11				;up
		brne	bout_cborder_3
		rcall	bout_nrhor
bout_cborder_3:	ret
			

;------------------------------------------------------------------------------
;note output
;------------------------------------------------------------------------------
bout_nrver:	rcall	bout_rver
		push	XL
		ldi	XL,30
		sbrc	tempreg6,0
		libmio_note
		pop	XL
		ret	

bout_nrhor:	rcall	bout_rhor
		push	XL
		ldi	XL,26
		sbrc	tempreg6,0
		libmio_note
		pop	XL
		ret	

bout_nbrick:	push	XL
		ldi	XL,55
		sbrc	tempreg6,0
		libmio_note
		pop	XL
		ret	
		
		
;------------------------------------------------------------------------------
;move ball temporary to X
;------------------------------------------------------------------------------
bout_mball:	lds	XL,bas_array+140		;X
		lds	XH,bas_array+141		;Y
		lds	YL,bas_array+142		;ball direction
		lds	YH,bas_array+143		;counter for dx

		sbrc	YL,2			;skip if up
		inc	XH			;row +1
		sbrs	YL,2
		dec	XH
		
		andi	YL,3			;mask dx bits		
		cpi	YL,0
		brne	bout_mball_1
		dec	XL
		ret

bout_mball_1:	cpi	YL,3
		brne	bout_mball_2
		inc	XL
		ret
		
bout_mball_2:	cpi	YL,1
		brne	bout_mball_3
		sbrc	YH,0
		dec	XL
		ret
			
bout_mball_3:	sbrc	YH,0
		inc	XL
		ret

;------------------------------------------------------------------------------
;undraw ball (3x3)
;------------------------------------------------------------------------------
bout_rball:
;delete old ball
		lds	YL,bas_array+140
		lds	YH,bas_array+141
		subi	YL,1				;X-1
		subi	YH,1				;Y-1
		movw	ZL,YL				;copy coords to second corner
		subi	ZH,0xfe				;+2
		subi	ZL,0xfe				;+2
		sts	libmio_color,const_0
		libmio_fbox
		ret

;------------------------------------------------------------------------------
;draw ball (3x3) to XH/XL
;------------------------------------------------------------------------------
bout_dball:
;draw ball
		sts	bas_array+140,XL		;store new X
	    	sts	bas_array+141,XH		;store new Y
		movw	YL,XL
		subi	YL,1				;X-1
		subi	YH,1				;Y-1
		movw	ZL,YL				;copy coords to second corner
		subi	ZH,0xfe				;+2
		subi	ZL,0xfe				;+2
		sts	libmio_color,const_1
		libmio_fbox
		ret
		
;------------------------------------------------------------------------------
;reflect at horizontal thing
;------------------------------------------------------------------------------				
bout_rhor:	lds	YL,bas_array+142
		subi	YL,4
		andi	YL,0x07				;limit to 0-7
		sts	bas_array+142,YL
		ret
		
;------------------------------------------------------------------------------
;reflect at vertical thing
;------------------------------------------------------------------------------				
bout_rver:	lds	YL,bas_array+142
		ldi	YH,3
		sub	YH,YL
		andi	YH,0x03
		andi	YL,0x04
		or	YL,YH
		sts	bas_array+142,YL
		ret		

;------------------------------------------------------------------------------
;reflect at diagonal thing
;------------------------------------------------------------------------------			
bout_rdia:	lds	YL,bas_array+142
		ldi	YH,7
		sub	YH,YL
		andi	YH,0x07
		sts	bas_array+142,YH
		ret		
		
						
;------------------------------------------------------------------------------
;draw brick no tempreg2
;------------------------------------------------------------------------------			
bout_bdraw:	ldi	YL,LOW(bas_array)
		ldi	YH,HIGH(bas_array)
		add	YL,tempreg2
		adc	YH,const_0
		ld	tempreg1,Y
		
		ldi	YL,16			;Xstart
		ldi	YH,18			;Ystart
		mov	ZL,tempreg2
		ldi	ZH,6
		andi	ZL,0x0f
		mul	ZH,ZL
		add	YL,r0
		mov	ZL,tempreg2	
		lsr	ZL
		lsr	ZL
		andi	ZL,0x1c
		add	YH,ZL
		movw	ZL,YL
		subi	ZH,0xfe			;+2 (dy=3)
		subi	ZL,0xfc			;+4 (dx=5)		
		sts	libmio_color,const_0	;inverse
		libmio_fbox			;delete this brick temporary
		cpi	tempreg1,0		;no brick
		brne	bout_bdraw_02
bout_bdraw_01:	ret		
		
bout_bdraw_02:	sts	libmio_color,const_1	;normal color
		cpi	tempreg1,1		;normal brick
		brne	bout_bdraw_03
		libmio_fbox			;draw normal
		rjmp	bout_bdraw_01
		
bout_bdraw_03:	libmio_box			;extended brick
		rjmp	bout_bdraw_01

		
		
;------------------------------------------------------------------------------		
; check ball on XH/XL	
; result in tr2 
; 0 1 2
; 7   3
; 6 5 4 	
;------------------------------------------------------------------------------		
bout_cball:	dec	XL			;P0
		dec	XH		
		rcall	bout_check		;check pos
		bld	tempreg2,0		;store result
		inc	XL			;P1
		rcall	bout_check		;check pos
		bld	tempreg2,1		;store result
		inc	XL			;P2		
		rcall	bout_check		;check pos
		bld	tempreg2,2		;store result
		inc	XH			;P3				
		rcall	bout_check		;check pos
		bld	tempreg2,3		;store result
		inc	XH			;P4
		rcall	bout_check		;check pos
		bld	tempreg2,4		;store result
		dec	XL			;P5
		rcall	bout_check		;check pos
		bld	tempreg2,5		;store result
		dec	XL			;P6								
		rcall	bout_check		;check pos
		bld	tempreg2,6		;store result
		dec	XH			;P7
		rcall	bout_check		;check pos
		bld	tempreg2,7		;store result		
		inc	XL			;restore pos
bout_cball_01:	ret
					
;------------------------------------------------------------------------------
; check brick on XH/XL
; T=result (1=touched)
;------------------------------------------------------------------------------		
bout_check:	push	XL			;save registers
		push	XH
		clt				;clear flag
		ldi	ZL,LOW(bout_htab*2-14)
		ldi	ZH,HIGH(bout_htab*2-14)			
		add	ZL,XL
		adc	ZH,const_0
		lpm	r16,Z			;hor result
		
		ldi	ZL,LOW(bout_vtab*2-10)
		ldi	ZH,HIGH(bout_vtab*2-10)			
		add	ZL,XH
		adc	ZH,const_0
		lpm	r17,Z			;vert result
		
		sbrs	r16,7			;skip if hbit
		rjmp	bout_check_e2		;no brick
		sbrs	r17,3			;skip if vbit
		rjmp	bout_check_e2		;no brick
		
		movw	tempreg7,r16
		ldi	YL,LOW(bas_array)	;table
		ldi	YH,HIGH(bas_array)	
		andi	r16,0x0f
		andi	r17,0x70
		add	r16,r17
		add	YL,r16
		adc	YH,const_0
		
		ld	r18,Y			;get value of brick
		cpi	r18,0
		breq	bout_check_e2		;do nothing here, is deleted
		
bout_check_01:	sbrc	r18,7			;is already done?
		rjmp	bout_check_e1		;do nothing here, is done
		
		dec	r18			;-1
		ori	r18,0x80		;set done flag
		st	Y,r18			;store back
		
		movw	XL,tempreg7
		andi	XL,0x0f
		andi	XH,0x70
		swap	XH
		ldi	YL,16			;Xstart
		ldi	YH,18			;Ystart
		ldi	ZL,6			;calc X
		mul	XL,ZL
		add	YL,r0		
				
		ldi	ZL,4			;calc Y
		mul	ZL,XH
		add	YH,r0
		
		movw	ZL,YL
		subi	ZH,0xfe			;+2 (dy=3)
		subi	ZL,0xfc			;+4 (dx=5)		
		sts	libmio_color,const_0	;inverse
		libmio_fbox			;delete this brick temporary		
		sts	libmio_color,const_1	;normal
		andi	r18,0xf			;type
		breq	bout_check_e1		;brick deleted

		cpi	r18,1			;normal brick
		brne	bout_check_02
		libmio_fbox			;draw normal
		rjmp	bout_check_e1
bout_check_02:	libmio_box			;extended brick		
				
bout_check_e1:	set				;set flag		
bout_check_e2:	pop	XH			;restore registers
		pop	XL
		ret				;thats all	
		
;------------------------------------------------------------------------------
;draw field
;------------------------------------------------------------------------------
bout_drawfield:	sts	libmio_color,const_1
		ldi	YL,11
		ldi	YH,10
		ldi	ZL,116
		ldi	ZH,63
		libmio_fbox
		sts	libmio_color,const_0
		ldi	YL,15
		ldi	YH,10
		ldi	ZL,111
		ldi	ZH,62
		libmio_fbox
		ldi	ZL,LOW(bout_l1*2)		;start leveltable
		ldi	ZH,HIGH(bout_l1*2)
		ldi	tempreg1,48			;bytes per level table
		lds	tempreg5,bas_array+150
		mul	tempreg1,tempreg5		;level
		add	ZL,r0
		adc	ZH,r1
		ldi	YL,LOW(bas_array)
		ldi	YH,HIGH(bas_array)
		ldi	XL,64
bout_dfield_01:	lpm	tempreg2,Z
		swap	tempreg2
		andi	tempreg2,0x0f
		st	Y+,tempreg2
		lpm	tempreg2,Z+
		andi	tempreg2,0x0f
		st	Y+,tempreg2
		dec	XL
		brne	bout_dfield_01
		
bout_drfield:		
bout_dfield_02:	ldi	tempreg2,0			;X start
bout_dfield_03:	rcall	bout_bdraw			;draw brick
		inc	tempreg2			;X+1
		cpi	tempreg2,96			;bricks per line
		brne	bout_dfield_03
		ret
		
;------------------------------------------------------------------------------
;move figure
;------------------------------------------------------------------------------
bout_mfig:	lds	YH,libmio_keycode
		lds	YL,bas_array+146
		cpi	YH,0xe2				;left
		brne	bout_mfig_3
		dec	YL
		cpi	YL,15
		breq	bout_mfig_2
bout_mfig_1:	sts	bas_array+146,YL
bout_mfig_2:	ret

bout_mfig_3:	cpi	YH,0xe3				;right
		brne	bout_mfig_2
		inc	YL		
		cpi	YL,104
		brcs	bout_mfig_1
		ret
	
;------------------------------------------------------------------------------
;draw figure
;------------------------------------------------------------------------------
bout_dfig:	lds	YL,bas_array+147		;old position
		lds	YH,bas_array+146		;new position
		cp	YL,YH				;moved?
		breq	bout_dfig_1			;branch if not
		ldi	YH,59
		movw	ZL,YL
		subi	ZH,0xff				;+1
		subi	ZL,0xf9				;+7
		sts	libmio_color,const_0
		libmio_fbox
	
		lds	YL,bas_array+146		;new position
		ldi	YH,59
		movw	ZL,YL
		subi	ZH,0xff				;+1
		subi	ZL,0xf9				;+7
		sts	libmio_color,const_1
		libmio_fbox
		sts	bas_array+147,YL		;copy position to old
bout_dfig_1:	ret				
				
;------------------------------------------------------------------------------
;colums for brick scheck
;bit 7=brick
;bit 6=left border
;bit 5=right border
;bit 0-3 brick number
;------------------------------------------------------------------------------			
bout_htab:    	.db	0x40,0x00			;pre space
		.db	0x80,0x80,0x80,0x80,0x80,0x00	;brick 1
		.db	0x81,0x81,0x81,0x81,0x81,0x00	;brick 2				                                
		.db	0x82,0x82,0x82,0x82,0x82,0x00	;brick 3
		.db	0x83,0x83,0x83,0x83,0x83,0x00	;brick 4		
		.db	0x84,0x84,0x84,0x84,0x84,0x00	;brick 5
		.db	0x85,0x85,0x85,0x85,0x85,0x00	;brick 6
		.db	0x86,0x86,0x86,0x86,0x86,0x00	;brick 7
		.db	0x87,0x87,0x87,0x87,0x87,0x00	;brick 8
		.db	0x88,0x88,0x88,0x88,0x88,0x00	;brick 9
		.db	0x89,0x89,0x89,0x89,0x89,0x00	;brick 10		
		.db	0x8a,0x8a,0x8a,0x8a,0x8a,0x00	;brick 11
		.db	0x8b,0x8b,0x8b,0x8b,0x8b,0x00	;brick 12		
		.db	0x8c,0x8c,0x8c,0x8c,0x8c,0x00	;brick 13
		.db	0x8d,0x8d,0x8d,0x8d,0x8d,0x00	;brick 14
		.db	0x8e,0x8e,0x8e,0x8e,0x8e,0x00	;brick 15
		.db	0x8f,0x8f,0x8f,0x8f,0x8f,0x00	;brick 16
		.db	0x20,0x00			;post space				
				                
;------------------------------------------------------------------------------
;rows for brick scheck
;bit 3=brick
;bit 2=upper border
;bit 1=lower border
;bit 0=me
;bit 6-4 brick row
;------------------------------------------------------------------------------			
bout_vtab:	.db	0x04,0x00,0x00,0x00,0x00,0x00,0x00,0x00 ;pre space		
		.db	0x08,0x08,0x08,0x00,0x18,0x18,0x18,0x00 ;1+2
		.db	0x28,0x28,0x28,0x00,0x38,0x38,0x38,0x00	;3+4
		.db	0x48,0x48,0x48,0x00,0x58,0x58,0x58,0x00	;5+6
		
		.db	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00	;free space
		.db	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00	;free space
		.db	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00	;free space		
		.db	0x00,0x00,0x00,0x01,0x01,0x00,0x00,0x02	;fig+lower border		

				
		
;------------------------------------------------------------------------------
;field data
;------------------------------------------------------------------------------
bout_l1:
;level 1
		.db	0x11,0x11,0x11,0x11,0x11,0x11,0x11,0x11
		.db	0x11,0x11,0x11,0x11,0x11,0x11,0x11,0x11
		.db	0x11,0x11,0x11,0x11,0x11,0x11,0x11,0x11
		.db	0x11,0x11,0x11,0x11,0x11,0x11,0x11,0x11
		.db	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
		.db	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
		
;level 2
		.db	0x11,0x11,0x11,0x11,0x11,0x11,0x11,0x11
		.db	0x11,0x11,0x11,0x11,0x11,0x11,0x11,0x11
		.db	0x11,0x11,0x11,0x11,0x11,0x11,0x11,0x11
		.db	0x11,0x11,0x11,0x11,0x11,0x11,0x11,0x11
		.db	0x11,0x11,0x11,0x11,0x11,0x11,0x11,0x11
		.db	0x11,0x11,0x11,0x11,0x11,0x11,0x11,0x11
;level 3
		.db	0x12,0x12,0x12,0x12,0x12,0x12,0x12,0x12
		.db	0x11,0x11,0x11,0x11,0x11,0x11,0x11,0x11
		.db	0x11,0x11,0x11,0x11,0x11,0x11,0x11,0x11
		.db	0x21,0x21,0x21,0x21,0x21,0x21,0x21,0x21
		.db	0x12,0x12,0x12,0x12,0x12,0x12,0x12,0x12
		.db	0x11,0x11,0x11,0x11,0x11,0x11,0x11,0x11
;level 4
		.db	0x11,0x40,0x41,0x11,0x11,0x14,0x04,0x11
		.db	0x11,0x40,0x41,0x00,0x00,0x14,0x04,0x11
		.db	0x11,0x40,0x41,0x03,0x30,0x14,0x04,0x11
		.db	0x11,0x40,0x41,0x00,0x00,0x14,0x04,0x11
		.db	0x11,0x40,0x41,0x11,0x11,0x14,0x04,0x11
		.db	0x11,0x11,0x11,0x11,0x11,0x11,0x11,0x11
		
	

													