;################################################################################
;#										#
;# avr-chipbasic2 - single chip basic computer with ATmega644			#
;# EEPROM/Flash memory interface						#
;# copyright (c) 2006-2008 Joerg Wolfram (joerg@jcwolfram.de)			#
;#										#
;#										#
;# This program is free software; you can redistribute it and/or		#
;# modify it under the terms of the GNU General Public License			#
;# as published by the Free Software Foundation; either version 2		#
;# of the License, or (at your option) any later version.			#
;#										#
;# This program is distributed in the hope that it will be useful,		#
;# but WITHOUT ANY WARRANTY; without even the implied warranty of		#
;# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the GNU		#
;# General Public License for more details.					#
;#										#
;# You should have received a copy of the GNU General Public			#
;# License along with this library; if not, write to the			#
;# Free Software Foundation, Inc., 59 Temple Place - Suite 330,			#
;# Boston, MA 02111-1307, USA.							#
;#										#
;################################################################################

		jmp	0x0000
;------------------------------------------------------------------------------
; write sysconf to eeprom
;------------------------------------------------------------------------------
mem_setconf:	lds	tempreg1,libmio_sysconf	;set number
		ldi	YL,LOW(2040)		;default -> config 
		ldi	YH,HIGH(2040)
		libeep_write
		ret

;------------------------------------------------------------------------------
; get sysconf
;------------------------------------------------------------------------------
mem_getconf:	ldi	YL,LOW(2040)		;default -> config 
		ldi	YH,HIGH(2040)
		libeep_read			;get value
		sts	libmio_sysconf,tempreg1
		ret		

;------------------------------------------------------------------------------
; set memory pointer to prog tempreg1
;------------------------------------------------------------------------------
mem_setprog:	push	ZH
		push	ZL	
		push	YH
		push	YL
		push	XL
		rcall	mem_progadr_0		;get ptr in Z
		sts	libmio_ram+44,ZL	;program base L
		sts	libmio_ram+45,ZH	;program base H
		pop	XL
		pop	YL
		pop	YH
		pop	ZL
		pop	ZH
		ret


mem_setthis:	ldi	ZL,LOW(bas_programs*2)	;base address
		ldi	ZH,HIGH(bas_programs*2)
		ldi	YL,LOW(3072)		;bytes/program
		ldi	YH,HIGH(3072)
		mov	XL,tempreg1
mem_setprog1:	cpi	XL,0
		breq	mem_setprog2		
		add	ZL,YL
		adc	ZH,YH
		dec	XL
		rjmp	mem_setprog1
mem_setprog2:	ret
						
;------------------------------------------------------------------------------
; read line (ctrl) from program memory to line buffer
;------------------------------------------------------------------------------
mem_readline:	lds	ZL,libmio_ram+44	;base address
		lds	ZH,libmio_ram+45
		ldi	XL,32			;bytes/line
		mul	XL,ctrl
		add	ZL,r0
		adc	ZH,r1
		adiw	ZL,32			;skip name
mem_readline0:	ldi	YL,LOW(bas_linebuf)	;buffer address
		ldi	YH,HIGH(bas_linebuf)
		ldi	XL,32			;no of bytes to copy
mem_readline1:	lpm	XH,Z+			;get from flash
		st	Y+,XH			;put to linebuffer
		dec	XL
		brne	mem_readline1
		ldi	XH,0xff			;append an extra EOL
		st	Y+,XH
		ret					

		
mem_spm:	in	tempreg1,SPMCSR
		sbrc	tempreg1,0
		rjmp	mem_spm
mem_spm1:	out	SPMCSR,XH
		spm
mem_spm2:	in	tempreg1,SPMCSR
		sbrc	tempreg1,0
		rjmp	mem_spm2
		ret
					
mem_waitrww:	in	tempreg1,SPMCSR
		sbrs	tempreg1,6
		ret
		ldi	XH,0x11
		rcall	mem_spm
		rjmp	mem_waitrww			


;------------------------------------------------------------------------------
; write line from Y to program memory
;------------------------------------------------------------------------------
mem_wrline:	cli
		mov	tempreg1,ZL		;pointer
		andi	tempreg1,0xe0		;mask out low bits
		brne	mem_wrline_1		;no first line
;erase page		
		ldi	XH,0x03			;page erase
		rcall	mem_spm
		rcall	mem_waitrww

mem_wrline_1:	ldi	XL,16			;16 words to copy
mem_wrline_2:	ld	r0,Y+			;low byte
		ld	r1,Y+			;high byte
		ldi	XH,0x01			;write to buffer
		rcall	mem_spm
		adiw	ZL,2			;ptr+2
		dec	XL			;loop counter
		brne	mem_wrline_2		;loop
;check for end of page						
		cpi	ZL,0x00			;end of page
		breq	mem_wrline_3		;yes
		ret
		
;write page
mem_wrline_3:	sbiw	ZL,2
		ldi	XH,0x05			;page write
		rcall	mem_spm
		rcall	mem_waitrww
		adiw	ZL,2
		ret
				

;------------------------------------------------------------------------------
; write array to program memory
;------------------------------------------------------------------------------
mem_writepage:	cli
mem_wpage0:	push	tempreg1
		push	XH
		push	XL
		ldi	XH,0x03			;page erase
		rcall	mem_spm
		rcall	mem_waitrww

		sbiw	ZL,2
		ldi	YL,LOW(bas_array+512)	;buffer address
		ldi	YH,HIGH(bas_array+512)
		ldi	XL,0x80			;no. of words to copy
mem_wpage1:	adiw	ZL,2
		ld	r0,Y+			;low byte
		ld	r1,Y+			;high byte
		ldi	XH,0x01			;write to buffer
		rcall	mem_spm
		dec	XL
		brne	mem_wpage1
	
		ldi	XH,0x05			;page write
		rcall	mem_spm
mem_wpage2:	rcall	mem_waitrww
		pop	XL
		pop	XH
		pop	tempreg1
		ret

;------------------------------------------------------------------------------
; read name from program memory to Y+
;------------------------------------------------------------------------------
mem_readname:	lds	ZL,libmio_ram+44	;base address of program
		lds	ZH,libmio_ram+45
		sbiw	ZL,32			;point to fname
mem_readname0:	ldi	XL,12			;bytes/name
mem_readname1:	lpm	XH,Z+		
		st	Y+,XH			;get bytes
		dec	XL
		brne	mem_readname1
		lds	XL,libmio_config
		sbrc	XL,7
		ret
		ldi	XL,LOW(libmio_cols*libmio_rows-12)	;offset
		ldi	XH,HIGH(libmio_cols*libmio_rows-12)
		add	YL,XL
		adc	YH,XH
		ldi	XL,12
		lds	XH,libmio_color
mem_readname2:	st	Y+,XH
		dec	XL
		brne	mem_readname2		
		ret					

;------------------------------------------------------------------------------
; read name from program ctrl to Y+
;------------------------------------------------------------------------------
mem_readnamec:	ldi	ZL,LOW(bas_programs*2)	;base address
		ldi	ZH,HIGH(bas_programs*2)
		ldi	XL,LOW(3072)		;bytes/program
		ldi	XH,HIGH(3072)
		mov	tempreg1,ctrl
mem_readnamec1:	cpi	tempreg1,0
		breq	mem_readname0
		add	ZL,XL
		adc	ZH,XH
		dec	tempreg1
		rjmp	mem_readnamec1			


;------------------------------------------------------------------------------
; calculate memory program address
;------------------------------------------------------------------------------
mem_progadr:	lds	tempreg1,libmio_lastmenu	;program to save
mem_progadr_0:	ldi	ZL,LOW(bas_programs*2)		;base
		ldi	ZH,HIGH(bas_programs*2)
		andi	tempreg1,0x03		;mask bits
		swap	tempreg1		;*16
		ldi	XL,0xc0			;*192
		mul	XL,tempreg1
		add	ZL,r0			;set ptr to start
		adc	ZH,r1			
		ret	

;------------------------------------------------------------------------------
; write screenshot to flash memory
;------------------------------------------------------------------------------
mem_scrs:	cli
		ldi	ZL,LOW(scrs_loc*2)
		ldi	ZH,HIGH(scrs_loc*2)
		ldi	YL,LOW(libmio_vram)
		ldi	YH,HIGH(libmio_vram)
		ldi	tempreg3,4		;4 pages
mem_scrs_1:	ldi	XH,0x03			;page erase
		rcall	mem_spm			;do command
		rcall	mem_waitrww		;wait for ready
		ldi	tempreg2,0x80		;no of words
		sbiw	ZL,2
mem_scrs_2:	adiw	ZL,2
		ld	r0,Y+			;low byte
		ld	r1,Y+			;high byte
		ldi	XH,0x01			;write to buffer
		rcall	mem_spm			;write data
		dec	tempreg2		;word counter
		brne	mem_scrs_2		;word loop
		ldi	XH,0x05			;page write
		rcall	mem_spm			;do command
		rcall	mem_waitrww		;wait for ready
		adiw	ZL,2			;start of next page	
		dec	tempreg3		;page counter
		brne	mem_scrs_1		;page loop
mem_scrs_3:	sbis	PINC,2	
		rjmp	mem_scrs_3
		ret
		
	